/*	Heterogeneous Grooming Optical Network Simulator (HEGONS)
	Suporting mixed routing & wavelength assignment algorithms and optional conversions
	Inputs/Outputs are clear once you run the program.
	Use -? to show help

	Code by: Ammar Muqaddas
	www.solostuff.net
	ammar@solostuff.net
	ammar.mu@gmail.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <search.h>
#include <ctype.h>
#include <malloc.h>
#include <memory.h>
#include <math.h>
#include <time.h>
#include "graphs.h"
#include "NDarrays.h"
#define MAX_FILE_NAME_SIZE 200
#define INFINITY 2147483647
#define UINFINITY 4294967295				// 2^32-1
#define INVALID 4294967295
#define DINVALID -1
#define DYNBUFF_SIZE 1000
#define CALC_ATTEMPTS
#define LIMIT_ATTEMPTS
#define AV_BP 1
#define OVERALL_BP 2
#define AOA_BP 3
#define BP_METHOD AV_BP
#define CHAN_TYPE unsigned long

// Gives the sign of a double
#define DSIGN(d1) (d1 < 0.0) ? -1 : ((d1 > 0.0) ? 1:0)


#undef BLOCK_IF_NOPATH

#undef TRY_PREV_CHANNEL

#define USE_PATH_NUM
/*	USE_PATH_NUM has to be defined if your going to use "Most Used RWA". If not then undefining
	USE_PATH_NUM will slightly increase the performance of other algorithms.
*/
#ifdef USE_PATH_NUM
	#define STORE_PATH_NUM(p1) p1->path_num = cu_path_num
#else
	#define STORE_PATH_NUM(p1)
#endif


#define LFREE_INDEX
#ifdef LFREE_INDEX 
	#define	GET_LAST(a1,a2) __min(a1,a2)
#else
	#define GET_LAST(a1,a2) a2
#endif

#undef FFREE_UPDATE
/*  If FFREE_UPDATE is defined, the RWA algorithms will update (increment) the 
	chstate[][].f_free (First Free Channel). This value acts as a hint for the algorithm.
	FFREE_UPDATE allowes a slightly better method to update this value resulting in
	a slightly lower "Average Number of attempts". In addition, It can lower blocking 
	probabilities when you use the -at option to impose a maximum limit on the 
	"Number of attempts" per call. 
	Note: FFREE_UPDATE option works best for "First Fit" RWA. 
*/

#undef LFREE_UPDATE
/*  If LFREE_UPDATE is defined, the RWA algorithms will update (decrement) the 
	chstate[][].l_free (Last Free Channel). This value acts as a hint for the algorithm.
	LFREE_UPDATE allowes a slightly better method to update this value resulting in
	a slightly lower "Average Number of attempts". In addition, It can lower blocking 
	probabilities when you use the -at option to impose a maximum limit on the 
	"Number of attempts" per call. 
*/

#undef UPDATE_FFREE
#define UPDATE_FFREE(a1) 
#ifdef FFREE_UPDATE
	#undef UPDATE_FFREE
	#define	UPDATE_FFREE(a1) if (chstate[y1][x1].f_free==(unsigned long)a1 && chstate[y1][x1].chan[a1]==0) chstate[y1][x1].f_free++
#endif

#undef UPDATE_LFREE
#define UPDATE_LFREE(a1)
#ifdef LFREE_UPDATE	
	#ifdef LFREE_INDEX 
		#undef UPDATE_LFREE
		#define	UPDATE_LFREE(a1) if (chstate[y1][x1].l_free==(long)a1 && chstate[y1][x1].chan[a1]==0) chstate[y1][x1].l_free--
	#endif
#endif

#undef FFREE
#undef LFREE
#define FFREE(y,x) chstate[y][x].f_free
#define LFREE(y,x) chstate[y][x].l_free


#undef FORCE_GROOM_TRAFFIC

#undef BREAK_ON_HOPS
#undef IN_PATH
#undef STORE_FIRST_FROM
#undef IS_ARRIVAL
#undef ARRIVAL_FROM
//#undef ARRIVAL_TO
#undef PATH_HOPS
#undef PATH_HOPS2
#undef CALC_PATH_HOPS2

// Those simple macros might be defined differently in latter versions.
// Some of them just do nothing for now. They are placeholders.
#define BREAK_ON_HOPS(n1)
#define IN_PATH(x,n1) (x!=nodes)
#define STORE_FIRST_FROM(p1)
#define IS_ARRIVAL() fr->etype==ARRIVAL
#define ARRIVAL_FROM() from
//#define ARRIVAL_TO(p1) p1->to
#define PATH_HOPS(v1) v1
#define DEFINE_PATH_HOPS2(v1)
#define CALC_PATH_HOPS2(v1,v2)
#define PATH_HOPS2(v1,v2) v2



#define USE_LLML_R
/*	defining USE_LLML_R enables Least Loaded Routing & Most Loaded Routing which recuires an extra
	overhead of a counter (tots_free) in the chan_state structure. "Most Loaded" routing is usually
	not defined in papers, we added it hear because it's just like "Least Loaded" with a slight modification
*/


// Event Types
#define ARRIVAL 0
#define DEPARTURE 1
#define HOLD 2
#define SYS_ARRIVAL 3

// RWA methods
#define NONE -1
#define FIRST_FIT 0
#define RANDOM_RWA 1
//#define LA_RWA 2
//#define EX_LA_RWA 3
//#define DLA_RWA 5
#define MU_RWA 6
//#define LMU_WA 7
#define LU_RWA 8
//#define LLU_WA 9
#define LL_R 10
#define ML_R 11


typedef struct _qevent
{	// Arrival: etype=0
	// Departure: etype=1

	double t;
	struct _qevent *link;
	//struct _qevent *blink;
	unsigned long from;
	unsigned long to;
	unsigned long *pchan;
	path *ppath;
	unsigned long nslots;
	short etype;
	
	// You can remove path_num if you don't want to use Indexed Lists
	unsigned long path_num;
	
} qevent;

typedef struct _sized_array
{	unsigned long *ar;
	unsigned long size;
} sized_array;

typedef struct _dsized_array
{	double *ar;
	unsigned long size;
} dsized_array;


typedef struct _chan_state
{	unsigned long *chan;
	unsigned long size;
	unsigned long f_free;

	#ifdef LFREE_INDEX
	long l_free;
	#endif

	#ifdef USE_LLML_R
	// Total number of free slots on link
	unsigned long tots_free;
	#endif

} chan_state;


typedef struct _node_capabilities
{	short conversions;
	short slot_inter;
	short node_type;
	sized_array attached_nodes;

	// Capacities outgoing from grooming ports
	long *attached_outcap;

	// Capacities incoming to grooming ports
	long *attached_incap;
	double groomout_prob;
	double groomin_prob;
} node_capabilities;

typedef struct _random_list
{	unsigned long *pr_list;
	unsigned long r_n,r_len;
	unsigned long forced_index;
} random_list;

typedef struct _ind_list_item
{	unsigned long item;
	long priority;
	struct _ind_list_item *link;
	struct _ind_list_item *blink;
} ind_list_item;

typedef struct _ind_list
{	ind_list_item *plist;
	ind_list_item *pfirst;
	unsigned long size;
	//unsigned long *pindex;
} ind_list;


#define LONG_PARAM 1
#define DOUBLE_PARAM 2
#define YESNO_PARAM 3

typedef struct _param_rang
{	double min,max;
	short type;
	short control;
} param_rang;


/*	Variable Descriptions -------------------
	
	Note: In all the code of the simulator, variable 'y' is always the source and variable 'x' is always the destination

	>> double Variables:

	cu_time: Holds the current time

	factor: A value to multiply all arrival rates by.
	
	cofactor: A value to divide all arrival rates by. 
	
	**load: 2D array that carries the arrival rates from reach source to destination

	drate: Connection departure rate.

	tot_attempts: Total Number of Wavelength assignment attmepts
	-----------

	>> unsigned long Variables: 
	
	**z: Array of randomly generated numbers for each source to destination pair.
	each time a call arrival is generated from y to x, a new number is put in z[y][x]

	node: Total number of nodes, including virtual nodes if they exist.

	real_nodes: Total number of nodes without the virtual nodes if they exist.
	
	zgen: keeps the previously generated random number.
	
	zgen2: keeps a second previously generated random number. zgen and zgen2 are completely
	different sequences used for different purposes

	chan_slots: Capacity of a wavelength, i.e the maximum number of time slots each wavelength
	can transmit concurrently

*/

unsigned long const a=16807,c=0,m=2147483648;
double const td=2.093;
double **load=NULL,cu_time,factor=1,cofactor=1,drate=1,tot_attempts=0;
//path **spaths;
unsigned long **z,nodes,real_nodes,zgen,zgen2,chan_slots,max_call_slots,min_call_slots,dif_call_slots,cu_path_num=0,temp1;
unsigned long max_path_size=INFINITY,min_path_size=0,max_paths=0,max_swi=INFINITY,min_swi=0,max_attempts=UINFINITY;
long **cap;
short r_method=NONE,wa_method=NONE,all_paths=0,virt_input=0,virt_output=0,local_sh=0,xy_cord=0;
chan_state **chstate;
sized_array ***minch_cap=NULL;
node_capabilities *ncapab;
qevent *fr;							// A pointer to the events linked list.
ind_list **pr_path_data=NULL;
ind_list pr_chan_data;
sized_paths **gpaths;
random_list rlist1,rlist2;


__inline double sqr(double a)
{	return a*a;
}

double Max2D(double **ar,unsigned long nodes,unsigned long *py,unsigned long *px,short diagonal)
{	// If diagonal==0 this function excludes the diagonal were y==x
	
	unsigned long y,x;
	double max=-INFINITY;
	
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			if ((diagonal || y!=x) && ar[y][x]>max)
			{	max=ar[y][x]; *py=y; *px=x;
			}
	return max;
}

double Min2D(double **ar,unsigned long nodes,unsigned long *py,unsigned long *px,short diagonal)
{	// If diagonal==0 this function excludes the diagonal were y==x
	
	unsigned long y,x;
	double min=INFINITY;
	
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			if ((diagonal || y!=x) && ar[y][x]<min)
			{	min=ar[y][x]; *py=y; *px=x;
			}
	return min;
}

double Average2D(double **ar,unsigned long nodes,short diagonal)
{	// If diagonal==0 this function excludes the diagonal were y==x
	
	unsigned long y,x;
	double sum=0;
	
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			if (diagonal || y!=x)
				sum += ar[y][x];

	return sum / (nodes*nodes-((diagonal) ? 0 : nodes));
} 

double Var(double *ar, unsigned long size,double av)
{	// Finding Variance
	
	unsigned long n;
	double sum=0;
	for (n=0; n<size; n++)
		sum=sum+(ar[n]-av)*(ar[n]-av);
		//sum=sum+ar[n]*ar[n];
	//return (sum-size*av*av)/(size-1);
	return sum/(size-1);
}

double Var2D(double **ar, unsigned long nodes,double av,short diagonal)
{	// Finding Variance of a 2D array
	// If diagonal==0 this function excludes the diagonal were y==x

	unsigned long size,y,x;
	double sum=0;

	size = nodes*nodes-nodes;
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			if (diagonal || y!=x)
				sum=sum+(ar[y][x]-av)*(ar[y][x]-av);
				//sum=sum+ar[y][x]*ar[y][x];
	//return (sum-size*av*av)/(size-1);
	return sum/(size-1);
}

double GenRand()
{	// Note: for our a,c,m values, it will never return exactly 0!!

	zgen=(a*zgen+c) % m; 
	return ((double) zgen/m); 
}

__inline double InGenRand()
{	// An inline version of GenRand
	// Note: for our a,c,m values, it will never return exactly 0!!

	zgen=(a*zgen+c) % m; 
	return ((double) zgen/m); 
}

unsigned long GetNextSeed()
{	return zgen;
}

void SetSeed(unsigned long seed)
{	zgen = seed;
}

__inline double InGenRand2()
{	/*	An other InGenRand. The only diference is that it uses zgen2 instead of
		zgen. This function is used in generating the number of time slots an arrival
		needs. If we used InGenRand() different RWA algorithms that also use InGenRand()
		will obviuosly interfere with the arrival of the calls. This will make comparison
		between algorithms harder.
		Note: for our a,c,m values, it will never return exactly 0!!
	*/

	zgen2=(a*zgen2+c) % m; 
	return ((double) zgen2/m); 
}

unsigned long GetNextSeed2()
{	return zgen2;
}

void SetSeed2(unsigned long seed)
{	zgen2 = seed;
}

double GenRandRange(double min, double max)
{	// Returns a random real number in the interval [min,max)
	// Note: for our a,c,m values, it will never return exactly min!!

	return (max-min)*InGenRand2()+min;
}

long GenIntRandRange(long min, long max)
{	// Returns a random integer number in the interval [min,max)
	// Note: for our a,c,m values, it will never return exactly min!!

	return (long) ((max-min+1)*InGenRand2()+min);
}

unsigned long DSizedArraysMaxSize(dsized_array *p,unsigned long size)
{	unsigned long n,max=0;

	for (n=0; n<size; n++) max=__max(max,p[n].size);
	return max;
}

unsigned long SizedArraysMaxSize(sized_array *p,unsigned long size)
{	unsigned long n,max=0;

	for (n=0; n<size; n++) max=__max(max,p[n].size);
	return max;
}


// -------------------------------------------------------------------------------
// CSV file printing functions
void PrintValueToCSV(FILE *pfile,double value,long index,const char *argv[])
{	
	if (pfile!=NULL)
	{	if (index>-1) { fprintf(pfile,"\n"); fprintf(pfile,&(argv[index][4])); }
		fprintf(pfile,", %g",value);
		fclose(pfile);
	}
}



// -------------------------------------------------------------------------------
// The following few functions are used with the ind_list (Indexed list) structure

void AllocateIndexedList(ind_list *pind_list,unsigned long size)
{	pind_list->size = size;
	if (size>0)
	{	pind_list->plist = (ind_list_item *) malloc(size*sizeof(*(pind_list->plist)));
		//pind_list->pindex = (unsigned long *) malloc(size*sizeof(*(pind_list->pindex)));
	} else
	{	pind_list->plist = NULL; //pind_list->pindex = NULL;
	}
}

void InitializeIndexedList(ind_list *pind_list)
{	unsigned long n;

	if (pind_list->size>0)
	{	pind_list->pfirst = pind_list->plist;
		pind_list->plist[0].blink = NULL;
		for (n=0; n<pind_list->size-1; n++)
		{	pind_list->plist[n].item = n;
			pind_list->plist[n].priority = 0;
			pind_list->plist[n].link = &(pind_list->plist[n+1]);
			if (n>0) pind_list->plist[n].blink = &(pind_list->plist[n-1]);
			//pind_list->pindex[n] = n;
		}
		pind_list->plist[n].item = n;
		pind_list->plist[n].priority = 0;
		pind_list->plist[n].link = NULL;
		if (n>0) pind_list->plist[n].blink = &(pind_list->plist[n-1]);
	}
}

void FreeIndexedList(ind_list *pind_list)
{	free(pind_list->plist);
}

#define DELINK_LIST_ITEM() \
	p2=pitem->blink; p=pitem->link; \
	if (p2!=NULL) \
		p2->link = p; \
	else \
		pind_list->pfirst = p; \
	if (p!=NULL) \
		p->blink = p2

#define RELINK_LIST_ITEM() \
	pitem->link = p; pitem->blink = p2; \
	if (p2!=NULL) p2->link = pitem; else pind_list->pfirst = pitem; \
	if (p!=NULL) p->blink = pitem

void (*DecreaseRListItem)(ind_list *pind_list, ind_list_item *pitem,long value);

void (*IncreaseRListItem)(ind_list *pind_list, ind_list_item *pitem,long value);

void (*DecreaseRListItemCaller)(qevent *p);

void (*DecreaseWAListItem)(ind_list *pind_list, ind_list_item *pitem,long value);

void (*IncreaseWAListItem)(ind_list *pind_list, ind_list_item *pitem,long value);

void DecreaseDesListItem(ind_list *pind_list, ind_list_item *pitem,long value)
{	ind_list_item *p, *p2;
	
	// Delink from the previous position
	DELINK_LIST_ITEM();

	// Find the new position
	pitem->priority -= value; //item = pitem->item; 
	while (p!=NULL)
	{	if (pitem->priority>=p->priority) break;
		p2=p; p = p->link;
	}

	// Relink to the new right position
	RELINK_LIST_ITEM();
}

void IncreaseDesListItem(ind_list *pind_list, ind_list_item *pitem,long value)
{	ind_list_item *p, *p2;
	
	// Delink from the previous position
	DELINK_LIST_ITEM();

	// Find the new position
	pitem->priority += value; //item = pitem->item; 
	while (p2!=NULL)
	{	if (pitem->priority<=p2->priority) break;
		p=p2; p2 = p2->blink;
	}

	// Relink to the new right position
	RELINK_LIST_ITEM();
}

void DecreaseAscListItem(ind_list *pind_list, ind_list_item *pitem,long value)
{	ind_list_item *p, *p2;
	
	// Delink from the previous position
	DELINK_LIST_ITEM();

	// Find the new position
	pitem->priority -= value; //item = pitem->item; 
	while (p2!=NULL)
	{	if (pitem->priority>=p2->priority) break;
		p=p2; p2 = p2->blink;
	}

	// Relink to the new right position
	RELINK_LIST_ITEM();
}

void IncreaseAscListItem(ind_list *pind_list, ind_list_item *pitem,long value)
{	ind_list_item *p, *p2;
	
	// Delink from the previous position
	DELINK_LIST_ITEM();

	// Find the new position
	pitem->priority += value; //item = pitem->item; 
	while (p!=NULL)
	{	if (pitem->priority<=p->priority) break;
		p2=p; p = p->link;
	}

	// Relink to the new right position
	RELINK_LIST_ITEM();
}


void DecreaseRListItemCallerNormal(qevent *p)
{	// Called by PopEvent functions using the DecreaseRListItemCaller pointer
	
	DecreaseRListItem(&(pr_path_data[p->from][p->to]),&(pr_path_data[p->from][p->to].plist[p->path_num]),p->nslots);
}
// End of ind_list (Indexed list) structure functions
// ---------------------------------------------------------------


/* ---------------------------------------------------------------------------------------
   The following three functions are used to randomize a list of sequencial numbers
   It's used buy RandFindChannels() to choose random channels efficiently
   Names: GenRandomList() and GetNextInList() and GetCurrentInList()
*/

// The following is a pointer to a function. By default it's initialized to GetNextInRList
// But some times we might need to make it point to ForceNextInRList.
unsigned long (*PGetNextInRList)(random_list *rlist);

__inline void GenRandomList(unsigned long from,unsigned long to,random_list *rlist)
{	// the randomized list will contain numbers between (from) and (to-1)
	
	rlist->r_len = to-from; free(rlist->pr_list);
	rlist->pr_list = malloc(rlist->r_len*sizeof(*(rlist->pr_list)));
	for (rlist->r_n=0; rlist->r_n<rlist->r_len; rlist->r_n++) rlist->pr_list[rlist->r_n] = from+rlist->r_n;  //**
	rlist->r_n=0;
}

__inline void FreeRandomList(random_list *rlist)
{	free(rlist->pr_list);
}

__inline unsigned long GetNextInRList(random_list *rlist)
{	
	unsigned long a1,a2;

	a1=(unsigned long) (InGenRand()*(rlist->r_len-rlist->r_n)+rlist->r_n);
	a2=rlist->pr_list[rlist->r_n]; rlist->pr_list[rlist->r_n]=rlist->pr_list[a1]; rlist->pr_list[a1]=a2;
	rlist->r_n++;
	return rlist->pr_list[rlist->r_n-1];	//**
}

__inline unsigned long ForceNextInRList(random_list *rlist)
{	/*	This funtion does the same as GetNextInRList with one difference that it does not
		generate a random index to choose from the rlist. The choice is forced to be rlist->forced_index
	*/
	unsigned long a2;

	PGetNextInRList = &GetNextInRList;
	a2=rlist->pr_list[rlist->r_n]; rlist->pr_list[rlist->r_n]=rlist->pr_list[rlist->forced_index]; rlist->pr_list[rlist->forced_index]=a2;
	rlist->r_n++;
	return rlist->pr_list[rlist->r_n-1];	//**
}

__inline unsigned long GetCurrentInRList(random_list *rlist)
{	return rlist->pr_list[rlist->r_n-1];

}
// ---------------------------------------------------------------------------------------


__inline void Init_chstate()
{	
	unsigned long x=0,y,n;

	for (y=0; y<nodes; y++)
	{	
		for (x=0; x<nodes; x++)
		{	
			#ifdef USE_LLML_R
				chstate[y][x].tots_free = chan_slots*chstate[y][x].size;
			#endif
			for (n=0; n<chstate[y][x].size; n++)
				chstate[y][x].chan[n] = chan_slots;
				chstate[y][x].f_free = 0;
				#ifdef LFREE_INDEX
					chstate[y][x].l_free = (long) chstate[y][x].size-1;
				#endif
		}
	}
}



// PopEvent functions --------------------------------------
void (* PopEvent)(qevent **p);

void NPopEvent(qevent **p)
{	qevent *p1;

	p1=(*p); (*p)=p1->link;
	//Blink: if ((*p)!=NULL) (*p)->blink=NULL; 
	if (p1->etype == DEPARTURE) free(p1->pchan); 
	free(p1);
}

void LuMuRWAPopEvent(qevent **p)
{	// For Least Used or Most Used Routing and Global Least Used or Most Used WA.
	// PopEvent & if it's a departure decrease all IndexedList items by fr->nslots
	
	qevent *p1;
	unsigned long n;

	p1=(*p); (*p)=p1->link;
	//Blink: if ((*p)!=NULL) (*p)->blink=NULL; 
	if (p1->etype == DEPARTURE) 
	{	// Be carefull since the following line may define "unsigned long a1"
		DEFINE_PATH_HOPS2(a1);

		DecreaseRListItemCaller(p1);
		CALC_PATH_HOPS2(a1,p1->ppath->size);
		for (n=0; n<PATH_HOPS2(a1,p1->ppath->size); n++)
			DecreaseWAListItem(&pr_chan_data,&(pr_chan_data.plist[p1->pchan[n]]),p1->nslots);
		free(p1->pchan); 
	}
	free(p1);
}

void LuMuRPopEvent(qevent **p)
{	// For Least Used or Most Used Routing.
	// PopEvent & if it's a departure decrease all IndexedList items by fr->nslots
	
	qevent *p1;

	p1=(*p); (*p)=p1->link;
	//Blink: if ((*p)!=NULL) (*p)->blink=NULL; 
	if (p1->etype == DEPARTURE) 
	{	DecreaseRListItemCaller(p1);
		free(p1->pchan); 
	}
	free(p1);
}

void LuMuWAPopEvent(qevent **p)
{	// For Glogal Least Used or Most Used WA.
	// PopEvent & if it's a departure decrease all IndexedList items by fr->nslots
	
	qevent *p1; 
	unsigned long n;

	p1=(*p); (*p)=p1->link;
	//Blink: if ((*p)!=NULL) (*p)->blink=NULL; 
	if (p1->etype == DEPARTURE) 
	{	// Be carefull since the following line may define "unsigned long a1"
		DEFINE_PATH_HOPS2(a1);

		CALC_PATH_HOPS2(a1,p1->ppath->size);
		for (n=0; n<PATH_HOPS2(a1,p1->ppath->size); n++)
			DecreaseWAListItem(&pr_chan_data,&(pr_chan_data.plist[p1->pchan[n]]),p1->nslots);
		free(p1->pchan); 
	}
	free(p1);
}

// Endof PopEvent functions -----------------------


void InSort(qevent **p,qevent *pn)
{	
	//Blink: qevent *bp=NULL;

	while ((*p)!=NULL && pn->t > (*p)->t)
	{	
		//Blink: bp=*p; 
		p=&((*p)->link);
	}

	pn->link = (*p); (*p)=pn;
	//Blink: pn->blink = bp;
	//Blink: if ((pn->link)!=NULL) (pn->link)->blink = pn;
}

void AllocateMinCapacitiesArrays(sized_paths **gpaths,unsigned long nodes)
{	
	unsigned long y,x;

	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			minch_cap[y][x] = (sized_array *) malloc(gpaths[y][x].size*sizeof(***minch_cap));
}

void GetPathMinCapacities(unsigned long y, path *ppath, sized_array *pmin_cap)
{	// y is the source node. (since it's not included in the path)

	unsigned long x1,y1,n1,n2,n3,min=UINFINITY;

	pmin_cap->size = ppath->size;
	pmin_cap->ar = malloc(pmin_cap->size*sizeof(*(pmin_cap->ar)));
	n1=0; n2=0; x1=y; min=UINFINITY;
	do 
	{	y1=x1; 
		x1=ppath->par[n1];
		if ((ncapab[y1].conversions || x1==nodes) && min!=UINFINITY)	//cap[y1][x1]<0
		{	for (n3=n2; n3<n1; n3++)
				pmin_cap->ar[n3] = min;
				n2=n1; min=UINFINITY;
		} 
		min = __min(min,chstate[y1][x1].size);
		n1++;
	} while (x1!=nodes);
}

long GetPathMinCapacity(unsigned long y, path *ppath)
{	unsigned long n,x,a1;
	long min=INFINITY;

	a1=ppath->size-EXTRA;
	for (n=0; n<a1; n++)
	{	x=ppath->par[n];
		min = __min(min,chstate[y][x].size);
		y=x;
	}
	return min;
}

int ComparePathsCosts(const path *ppath1,const path *ppath2)
{	return ppath1->tot_cost - ppath2->tot_cost;
}

int RevComparePathsCosts(const path *ppath1,const path *ppath2)
{	return ppath2->tot_cost - ppath1->tot_cost;

	//return ((double) rand()/RAND_MAX)*3-1;
}


void AddPartialNodesPaths(unsigned long y, unsigned long x, unsigned long num)
{	/* Add all combinations of self-cycle paths for partial grooming nodes
	   Note: pcu_path holds the current path (Including the first node). So we have
	   to exclude the first node when we add the path. */

	static path pcu_path, *ppath;
	static long depth = -2;
	unsigned long n,cu_node,prev_node,a1;
	short flag1;

	if (depth==-2)
	{	ppath = &(gpaths[y][x].ppaths[num]);
		pcu_path.size = 0; pcu_path.tot_cost = 0;
		// Prepare maximum path size possible
		pcu_path.par = malloc((3*ppath->size+3+EXTRA)*sizeof(*(pcu_path.par)));
	}

	depth++;
	if (depth>-1)
		cu_node = ppath->par[depth]; else cu_node = y;
	flag1 = (ncapab[cu_node].node_type==4) && (depth!=(long) ppath->size-1);
	if (depth>0) prev_node = ppath->par[depth-1]; else prev_node = y;

	// Check that previous node is not already an attached node (G-fabric)
	if (depth>=0)
		for (n=0; n<ncapab[cu_node].attached_nodes.size; n++)
		{	flag1 = (flag1 && prev_node!=ncapab[cu_node].attached_nodes.ar[n]);
			if (!flag1) break;
		}

	// Check that next node is not already an attached node (G-fabric)
	if (flag1 && depth<(long) ppath->size-1)
		for (n=0; n<ncapab[cu_node].attached_nodes.size; n++)
		{	flag1 = (flag1 && ppath->par[depth+1]!=ncapab[cu_node].attached_nodes.ar[n]);
			if (!flag1) break;
		}


	// Place the core node itself in the path
	pcu_path.par[pcu_path.size] = cu_node; pcu_path.size++;
	if (depth<(long) ppath->size-1)
		AddPartialNodesPaths(y,x,num);
	else
	{	// Add a new path (except the original path)	
		a1 = pcu_path.size-1-ppath->size;
		if (a1!=0 && a1<=2*max_swi && a1>=2*min_swi)
		{	gpaths[y][x].size++;
			gpaths[y][x].ppaths = realloc(gpaths[y][x].ppaths,gpaths[y][x].size*sizeof(*(gpaths[y][x].ppaths)));
			ppath = &(gpaths[y][x].ppaths[num]);
			#if EXTRA==1
				pcu_path.par[pcu_path.size] = nodes;
			#endif
			pcu_path.par++; pcu_path.size--;
			ClonePath(&pcu_path,&(gpaths[y][x].ppaths[gpaths[y][x].size-1]));
			pcu_path.par--; pcu_path.size++;
		}
	}
	pcu_path.size--;

	// Self-cycle core node with it's attahced nodes one by one.
	if (flag1)
	{	pcu_path.par[pcu_path.size] = cu_node;
		pcu_path.par[pcu_path.size+2] = cu_node;
		pcu_path.size += 3;
		for (n=0; n<ncapab[cu_node].attached_nodes.size; n++)
		{	pcu_path.par[pcu_path.size-2] = ncapab[cu_node].attached_nodes.ar[n];
			if (depth<(long) ppath->size-1)
				AddPartialNodesPaths(y,x,num);
			else
			{	// Add a new path
				a1 = pcu_path.size-1-ppath->size;
				if (a1<=2*max_swi && a1>=2*min_swi)
				{	gpaths[y][x].size++;
					gpaths[y][x].ppaths = realloc(gpaths[y][x].ppaths,gpaths[y][x].size*sizeof(*(gpaths[y][x].ppaths)));
					ppath = &(gpaths[y][x].ppaths[num]);
					#if EXTRA==1
						pcu_path.par[pcu_path.size] = nodes;
					#endif
					pcu_path.par++; pcu_path.size--;				
					ClonePath(&pcu_path,&(gpaths[y][x].ppaths[gpaths[y][x].size-1]));
					pcu_path.par--; pcu_path.size++;
				}
			}
		}
		pcu_path.size -= 3;
	}

	depth--;

	if (depth==-2)
		free(pcu_path.par);
}

__inline void GenArrival(unsigned long y, unsigned long x)
{		
	qevent *p1;

	z[y][x]=(a*z[y][x]+c) % m;
	p1 = malloc(sizeof(qevent)); 
	p1->link=NULL; p1->t = cu_time - log(1-((double) z[y][x]/m))/load[y][x]; 
	p1->etype=ARRIVAL; p1->from = y; p1->to = x; // p1->pchan = NULL;
	STORE_FIRST_FROM(p1);

	#ifndef FORCE_GROOM_TRAFFIC
		if (ncapab[y].slot_inter)
			p1->nslots = (unsigned long) (InGenRand2()*dif_call_slots+min_call_slots);		//**
		else 
			p1->nslots = chan_slots;
	#else
		p1->nslots = (unsigned long) (InGenRand2()*dif_call_slots+min_call_slots);	
	#endif
	InSort(&fr,p1);
}

__inline void GenArrivalAtExt(unsigned long y, unsigned long x,double t,short etype,unsigned long first_from)
{		
	qevent *p1;

	p1 = malloc(sizeof(qevent)); 
	p1->link=NULL; p1->t = t;
	p1->etype=etype; p1->from = y; p1->to = x; // p1->pchan = NULL;

	#ifndef FORCE_GROOM_TRAFFIC
		if (ncapab[y].slot_inter)
			p1->nslots = (unsigned long) (InGenRand2()*dif_call_slots+min_call_slots);		//**
		else 
			p1->nslots = chan_slots;
	#else
		p1->nslots = (unsigned long) (InGenRand2()*dif_call_slots+min_call_slots);	
	#endif
	InSort(&fr,p1);
}

__inline qevent *GenDeparture(unsigned long y,unsigned long x,unsigned long *pchan, path *ppath,unsigned long nslots)
{		
	qevent *p1;

	z[y][x]=(a*z[y][x]+c) % m;
	p1 = malloc(sizeof(qevent)); 
	p1->link=NULL; p1->t = cu_time-log(1-((double) z[y][x]/m))/drate; 
	p1->etype=DEPARTURE; p1->from = y; p1->to = x; p1->pchan = pchan;
	p1->nslots = nslots; p1->ppath = ppath; STORE_PATH_NUM(p1); //STORE_LASTTO(p1);
	InSort(&fr,p1); return p1;
}

__inline qevent *GenDepartureExt(unsigned long y,unsigned long x,unsigned long *pchan, path *ppath,unsigned long nslots, unsigned long first_from)
{		
	qevent *p1;

	z[y][x]=(a*z[y][x]+c) % m;
	p1 = malloc(sizeof(qevent)); 
	p1->link=NULL; p1->t = cu_time-log(1-((double) z[y][x]/m))/drate; 
	p1->etype=DEPARTURE; p1->from = y; p1->to = x; p1->pchan = pchan;
	p1->nslots = nslots; p1->ppath = ppath; STORE_PATH_NUM(p1); 
	InSort(&fr,p1); return p1;
}

/* __inline qevent *GenDepartureAt(unsigned long y, unsigned long x, unsigned long *pchan, path *ppath, unsigned long nslots, double t)
{		
	qevent *p1;

	p1 = malloc(sizeof(qevent)); 
	p1->link=NULL; p1->t = t; 
	p1->etype=DEPARTURE; p1->from = y; p1->to = x; p1->pchan = pchan;
	p1->nslots = nslots; p1->ppath = ppath; STORE_PATH_NUM(p1); //STORE_LASTTO(p1);
	InSort(&fr,p1); return p1;
} */

__inline qevent *GenDepartureAtExt(unsigned long y, unsigned long x, unsigned long *pchan, path *ppath, unsigned long nslots, double t, unsigned long first_from)
{		
	qevent *p1;

	p1 = malloc(sizeof(qevent)); 
	p1->link=NULL; p1->t = t; 
	p1->etype=DEPARTURE; p1->from = y; p1->to = x; p1->pchan = pchan;
	p1->nslots = nslots; p1->ppath = ppath; STORE_PATH_NUM(p1); //STORE_LASTTO(p1);
	InSort(&fr,p1); return p1;
}

/* 
	#define GenDepartureM(from,to,pchan,ppath,nslots,first_from) GenDeparture(from,to,pchan,ppath,nslots);
	#define GenDepartureAtM(from,to,pchan,ppath,nslots,first_from) GenDeparture(from,to,pchan,ppath,nslots);
#endif */

__inline void ReserveChannels(unsigned long from,unsigned long to,unsigned long *pchan, path *ppath,unsigned long nslots)
{	unsigned long n=0,y=from,x;

	x=ppath->par[n];
	do
	{	
		chstate[y][x].chan[pchan[n]] -= nslots;
		#ifdef USE_LLML_R
			chstate[y][x].tots_free -= nslots;
		#endif


		if (chstate[y][x].chan[pchan[n]]==0)
		{	if (chstate[y][x].f_free==pchan[n]) chstate[y][x].f_free++;
			#ifdef LFREE_INDEX
				if (chstate[y][x].l_free == (long)pchan[n]) chstate[y][x].l_free--;
			#endif
		}
		y=x; n++; x=ppath->par[n];
	} while IN_PATH(x,n);
}

__inline void FreeChannels(unsigned long from,unsigned long to,unsigned long *pchan,path *ppath,unsigned long nslots)
{	// Frees the channels reserved by the first event in the queue (list)

	unsigned long n,y,x;
	
	n=0; y=from; x=ppath->par[n];
	do
	{	
		chstate[y][x].chan[pchan[n]] += nslots;
		#ifdef USE_LLML_R
			chstate[y][x].tots_free += nslots;
		#endif
		if (pchan[n]<chstate[y][x].f_free) chstate[y][x].f_free=pchan[n];
		#ifdef LFREE_INDEX
			if ((long) pchan[n]>chstate[y][x].l_free) chstate[y][x].l_free=(long) pchan[n];
		#endif

		y=x; n++; x=ppath->par[n];
	} while IN_PATH(x,n);
}


/* -----------------------------------------------------------------------------------------
   Routing functions
   The routing functions are actually part of this file but I seperated them in "routing.c"
   for convenience. Therefore "routing.c" should not be compiled since it will be automatically 
   compiled with this "Hegons.c" file.
*/
#include "routing.c"



// -----------------------------------------------------------------------------------------
// Channel (Wavelength Assignment) Finding functions

#ifndef MIXED_RWA
	#undef ChooseFirstPath
	#undef ChoosePath
	#undef UpdatePathSuccess
	#define ChooseFirstPath(y,x) FFChooseFirstPath(y,x)
	#define ChoosePath() FFChoosePath()
	#define UpdatePathSuccess(req_slots) FFUpdatePathSuccess(req_slots)
#endif

short FFFindChannels(unsigned long y,unsigned long x,unsigned long **pchan, path **ppath,unsigned long req_slots)
{	
	/* (*pchan) should be initilized to NULL before the function is called 
	   (unless you know what you are doing!!)
	*/
	
	unsigned long n,n2,n3,path_n,y1,x1;
	long n1,a2;
	short flag; 
	#ifdef TRY_PREV_CHANNEL
	short is_prevch;
	#endif
	#ifdef LIMIT_ATTEMPTS
	unsigned long attempts=0;
	#endif


	#ifdef BLOCK_IF_NOPATH
		if (!gpaths[y][x].size) return 0;
	#endif

	// path_n is the path number
	path_n=ChooseFirstPath(y,x);

Newpath:

	(*pchan) = (unsigned long *) realloc(*pchan,PATH_HOPS(gpaths[y][x].ppaths[path_n].size)*sizeof(**pchan));
	n=0; n2=n; y1=y; x1=gpaths[y][x].ppaths[path_n].par[n]; 
	#ifdef TRY_PREV_CHANNEL
		is_prevch=0; n1=FFREE(y1,x1);
	#endif

	a2 = GET_LAST(LFREE(y1,x1)+1,(long) minch_cap[y][x][path_n].ar[n]);
	do
	{	flag=1; 
		#ifndef TRY_PREV_CHANNEL
			n1=FFREE(y1,x1);
			//printf(" n1=%ld",n1);
		#endif
		while (n1<a2)
		{	
			#ifdef LIMIT_ATTEMPTS
				if (attempts>=max_attempts) { free(*pchan); return 0; }
				attempts++;
			#endif
			#ifdef CALC_ATTEMPTS
				tot_attempts++;
			#endif

			do
			{	
				if (ncapab[y1].slot_inter && ncapab[x1].slot_inter)
					flag = (chstate[y1][x1].chan[n1]<req_slots);
				else flag = (chstate[y1][x1].chan[n1]<req_slots || chstate[y1][x1].chan[n1]!=chan_slots);
				if (flag) break;				

				n2++; y1=x1; x1=gpaths[y][x].ppaths[path_n].par[n2];
				BREAK_ON_HOPS(n2);
				
			// While Checks for conversions on next link.
			} while (x1!=nodes && !ncapab[y1].conversions);		// cap[y1][x1]>0
				
			if (flag)						
			{	UPDATE_FFREE(n1); UPDATE_LFREE(n1);
				if (n==0) y1=y; else y1=gpaths[y][x].ppaths[path_n].par[n-1];
				x1=gpaths[y][x].ppaths[path_n].par[n]; n2=n;
				#ifdef TRY_PREV_CHANNEL	
					if (is_prevch) { n1 = FFREE(y1,x1)-1; is_prevch=0; }
				#endif
			} else 
			{	
				for (n3=n; n3<n2; n3++)
					(*pchan)[n3] = n1;
				break;
			}
			n1++;
		}
		BREAK_ON_HOPS(n2);
		
		// If there is no free channel in partial path, select the next path and try again
		if (flag) 
		{	
			path_n=ChoosePath(); if (path_n!=INVALID) goto Newpath; else { free(*pchan); return 0; }
		}
			
		n=n2; 
		a2 = GET_LAST(LFREE(y1,x1)+1,(long) minch_cap[y][x][path_n].ar[n]);
		#ifdef TRY_PREV_CHANNEL	
			if (n1<a2) is_prevch=1;
			else { n1 = FFREE(y1,x1); is_prevch=0; }
		#endif
	} while (x1!=nodes);
	
	(*ppath) = &(gpaths[y][x].ppaths[path_n]); 
	UpdatePathSuccess(req_slots); return 1;
	
}


#ifndef MIXED_RWA
	#undef ChooseFirstPath
	#undef ChoosePath
	#undef UpdatePathSuccess
	#define ChooseFirstPath(y,x) RandChooseFirstPath(y,x)
	#define ChoosePath() RandChoosePath()
	#define UpdatePathSuccess(req_slots) RandUpdatePathSuccess(req_slots)
#endif

short RandFindChannels(unsigned long y,unsigned long x,unsigned long **pchan, path **ppath,unsigned long req_slots)
{	
	/* (*pchan) should be initilized to NULL before the function is called 
	   (unless you know what you are doing!!)
	*/
	
	unsigned long n,n1,n2,n3,n4,path_n,a1,a2,y1,x1;
	short flag;
	#ifdef LIMIT_ATTEMPTS
	unsigned long attempts=0;
	#endif

	#ifdef BLOCK_IF_NOPATH
		if (!gpaths[y][x].size) return 0;
	#endif
	// path_n is the path number
	path_n = ChooseFirstPath(y,x);

Newpath2:

	(*pchan) = (unsigned long *) realloc(*pchan,PATH_HOPS(gpaths[y][x].ppaths[path_n].size)*sizeof(**pchan));
	n=0; n2=n; 
	#ifdef TRY_PREV_CHANNEL
		n4=UINFINITY;
	#endif
	y1=y; x1=gpaths[y][x].ppaths[path_n].par[n];
	do
	{	flag=1;
		a1=FFREE(y1,x1); 
		a2 = GET_LAST((unsigned long) LFREE(y1,x1)+1,minch_cap[y][x][path_n].ar[n]);	
		if (a1<a2) GenRandomList(a1,a2,&rlist1);
		#ifdef TRY_PREV_CHANNEL
			if (n4>=a1 && n4<a2) 
			{	rlist1.forced_index=n4-a1; PGetNextInRList = &ForceNextInRList;
			}
		#endif
		for (n1=a1; n1<a2; n1++)
		{	
			#ifdef LIMIT_ATTEMPTS
				if (attempts>=max_attempts) { free(*pchan); return 0; }
				attempts++;
			#endif
			#ifdef CALC_ATTEMPTS 
				tot_attempts++;
			#endif

			n4 = PGetNextInRList(&rlist1);
			do
			{	
				if (ncapab[y1].slot_inter && ncapab[x1].slot_inter)
					flag = (chstate[y1][x1].chan[n4]<req_slots);
				else flag = (chstate[y1][x1].chan[n4]<req_slots || chstate[y1][x1].chan[n4]!=chan_slots);
				if (flag) break;

				n2++; y1=x1; x1=gpaths[y][x].ppaths[path_n].par[n2];
				BREAK_ON_HOPS(n2);
				
			// While Checks for conversions on next node.
			} while (x1!=nodes && !ncapab[y1].conversions);		// cap[y1][x1]>0
			
			if (flag)
			{	UPDATE_FFREE(n4); UPDATE_LFREE(n4);
				if (n==0) y1=y; else y1=gpaths[y][x].ppaths[path_n].par[n-1];
				x1=gpaths[y][x].ppaths[path_n].par[n]; n2=n;
			} else 
			{	
				for (n3=n; n3<n2; n3++)
					(*pchan)[n3] = n4;
				break;
			}
		}
		BREAK_ON_HOPS(n2);

		// If there is no free channel in partial path, free arrays and return 0
		if (flag) 
		{	
			path_n=ChoosePath(); if (path_n!=INVALID) goto Newpath2;
			else { free(*pchan); return 0; }
		}
		
		n=n2;
	} while (x1!=nodes);
	
	(*ppath) = &(gpaths[y][x].ppaths[path_n]); 
	UpdatePathSuccess(req_slots); return 1;
} 



#ifndef MIXED_RWA
	#undef ChooseFirstPath
	#undef ChoosePath
	#undef UpdatePathSuccess
	#define ChooseFirstPath(y,x) LUMUChooseFirstPath(y,x)
	#define ChoosePath() LUMUChoosePath()
	#define UpdatePathSuccess(req_slots) LUMUUpdatePathSuccess(req_slots)
#endif

short LUMUFindChannels(unsigned long y,unsigned long x,unsigned long **pchan, path **ppath,unsigned long req_slots)
{	/* Uses (Global) Least Used or Most Used WA. (This function can do both LU and MU !!)
	   (*pchan) should be initilized to NULL before the function is called 
	   (unless you know what you are doing!!)
	*/

	unsigned long n,n1,n2,n3,path_n,a2,y1,x1;
	short flag; 
	ind_list_item *pi2;
	#ifdef TRY_PREV_CHANNEL
	short is_prevch;
	#endif
	#ifdef LIMIT_ATTEMPTS
	unsigned long attempts=0;
	#endif


	#ifdef BLOCK_IF_NOPATH
		if (!gpaths[y][x].size) return 0;
	#endif
	// path_n is the path number, n6 is the path index in MU list
	path_n = ChooseFirstPath(y,x);

Newpath:

	(*pchan) = (unsigned long *) realloc(*pchan,PATH_HOPS(gpaths[y][x].ppaths[path_n].size)*sizeof(**pchan));
	n=0; n2=n; y1=y; x1=gpaths[y][x].ppaths[path_n].par[n]; 
	#ifdef TRY_PREV_CHANNEL
		is_prevch=0; pi2 = pr_chan_data.pfirst; n1=pi2->item;
	#endif

	a2 = GET_LAST(LFREE(y1,x1)+1,(long) minch_cap[y][x][path_n].ar[n]);
	do
	{	flag=1; 
		#ifndef TRY_PREV_CHANNEL
			pi2 = pr_chan_data.pfirst;
		#endif
		do
		{	n1=pi2->item;
			if (n1>=FFREE(y1,x1) && n1<a2)  //**
			{	
				#ifdef LIMIT_ATTEMPTS
					if (attempts>=max_attempts) { free(*pchan); return 0; }
					attempts++;
				#endif
				#ifdef CALC_ATTEMPTS
					tot_attempts++;
				#endif

				do
				{	
					if (ncapab[y1].slot_inter && ncapab[x1].slot_inter)
						flag = (chstate[y1][x1].chan[n1]<req_slots);
					else flag = (chstate[y1][x1].chan[n1]<req_slots || chstate[y1][x1].chan[n1]!=chan_slots);
					if (flag) break;
					n2++; y1=x1; x1=gpaths[y][x].ppaths[path_n].par[n2];

					BREAK_ON_HOPS(n2);
				// While Checks for conversions on next link.
				} while (x1!=nodes && !ncapab[y1].conversions);		// cap[y1][x1]>0
					
				if (flag)						
				{	UPDATE_FFREE(n1); UPDATE_LFREE(n1);
					if (n==0) y1=y; else y1=gpaths[y][x].ppaths[path_n].par[n-1];
					x1=gpaths[y][x].ppaths[path_n].par[n]; n2=n;
					#ifdef TRY_PREV_CHANNEL	
						if (is_prevch) { pi2 = pr_chan_data.pfirst; is_prevch=0; }
						else pi2=pi2->link;
					#else
						pi2=pi2->link;
					#endif
				} else 
				{	
					for (n3=n; n3<n2; n3++)
						(*pchan)[n3] = n1;
					break;
				}
			} else pi2=pi2->link;
		} while (pi2!=NULL);
		BREAK_ON_HOPS(n2);

		// If there is no free channel in partial path, select the next path and try again
		if (flag) 
		{	
			path_n=ChoosePath(); if (path_n!=INVALID) goto Newpath; else { free(*pchan); return 0; }
		}
			
		n=n2; 
		a2 = GET_LAST(LFREE(y1,x1)+1,(long) minch_cap[y][x][path_n].ar[n]);
		#ifdef TRY_PREV_CHANNEL	
			if (n1>=FFREE(y1,x1) && n1<a2) is_prevch=1; //**
			else { pi2 = pr_chan_data.pfirst; is_prevch=0; }
		#endif
	} while (x1!=nodes);
	
	(*ppath) = &(gpaths[y][x].ppaths[path_n]); 

	// Update LU/MU lists
	CALC_PATH_HOPS2(a2,(*ppath)->size);
	for (n3=0; n3<PATH_HOPS2(a2,(*ppath)->size); n3++)
		IncreaseWAListItem(&pr_chan_data,&(pr_chan_data.plist[(*pchan)[n3]]),req_slots);
	UpdatePathSuccess(req_slots); return 1;	
}


void PrintUsage(short ex_flag)
{
	printf("\nHegons v0.4 Heterogeneous Grooming Optical Network Simulator \nCopyright (c) 2005-2007 Ammar Muqaddas\n");
	if (ex_flag)
		printf("\nUsage: Hegons [-ff|-r|-mu|-lu|-ll|-ml] [-s:seed] [-b:value] [-t:time] [-d:drate] [-m] [-a] [-mx:length] [-mn:length] [-mp:number] [-px:number] [-pn:number] [-vi] [-vo] [-lo] [-of:prefix] [-or:name] [-at:number] <-nt:netfile | -i>");
	else
		printf("\nUsage: Hegons [-options...] <-nt:netfile | -i>");
	printf(" [-?]\n\n");
	printf("  -ff		Use first fit RWA. This is the default method\n");
	printf("  -r		Use random RWA\n");
	printf("  -mu		Use Most Used Routing and Global channels Most Used WA\n");
	printf("  -lu		Use Least Used Routing and Global channels Least Used WA\n");
	printf("  -ll		Use Least Loaded Routing. When used for WA, first fit is used.\n");
	printf("  -ml		Use Most Loaded Routing. When used for WA, first fit is used.\n");
	printf("  -s:seed	Randomization Seed. If omited, system timer is used\n");
	printf("  -b:value	Number of batches. i.e, number of times the simulation is run.\n");
	printf("		If omited, It will be read from the network file specified by\n");
	printf("		-nt or requested interactivly if -i is used.\n");
	printf("  -t:time	Total time (in time units) each batch should run.\n");
	printf("  -d:drate	Common departure rate. Default = 1\n");
	printf("  -f:factor	A factor that the arrival rates matrix will be multiplied by\n");
	printf("  -c:cofactor	A cofactor that the arrival rates matrix will be devided by\n");					
	printf("  -a		Forces the simulator to extract all possible paths\n");
	printf("  -mx:length	Specifies a maximum length for each path generated.\n");
	printf("  -mn:length	Specifies a minimum length for each path generated.\n");
	printf("  -mp:number	Specifies the maximum number of paths to be generated for\n		for each source to destination pair. If omited all required\n		paths are generated.\n");
	printf("  -px:number	Specifies the maximum number of G-Fabric switchings allowed\n		for any path that contains partial grooming nodes\n");
	printf("  -pn:number	Specifies the minimum number of G-Fabric switchings allowed\n		for any path that contains partial grooming nodes. But\n		No G-Fabric Switchings (Switchings=0) is always applied.\n");
	printf("  -vi		Prompt to enter capacities and arrival rates for virtual nodes\n");
	printf("  -vo		Show blocking probabilities of virtual nodes\n");
	printf("  -lo		Include local calls in simulation results. i.e. when x=y\n");
	printf("  -of:prefix	Specifies a prefix name for output CSV file names. The\n		results are saved into those files.\n");
	printf("  -or:name	Specifies a row name to be writen into the output CSV files\n		This option is used with the -of option.\n");
	printf("  -i		Interactive mode. The user is prompted for the network details.\n");
	printf("  -nt:file	Specifies a network topology/info file to read from. The file\n		will be overwriten if -g option is used.\n");
	printf("  -at:number	Specifies the maximum number of attempts for a call before\n		it is force blocked\n");

	printf("  -?		Show this usage info with extended options view.\n");
	#ifdef MIXED_RWA
		printf("\nNote: Mixed RWA is enabled. You can use different routing and WA methods by\nspecifying two RWA methods. For example, If you enter -r -mu. The simulator\nwill use Random routing and Most Used WA. The first method entered is always\nfor routing, and the second is for WA\n");
	#endif
}

void CheckParamCorrectness()
{	// Check some user input for invalid usage

	if (r_method==NONE) r_method=FIRST_FIT;
	if (wa_method==NONE) wa_method=r_method;
	if (wa_method==LL_R) wa_method=FIRST_FIT;
	if (wa_method==ML_R) wa_method=FIRST_FIT;
	#ifndef MIXED_RWA
		if (r_method!=wa_method) 
		{	printf("\nYou should #define MIXED_RWA compiler directive to use mixed RWA\n"); exit(0); }
	#endif

	// Check if -at is issued with defining LIMIT_ATTEMPTS
	#ifndef LIMIT_ATTEMPTS
		if (max_attempts!=UINFINITY) 
		{	printf("\nYou should #define LIMIT_ATTEMPTS to be able to use the -at option.\n"); exit(0);
		}
	#endif

}
// ---------------------------------------------------------------------------------


void main(int argc, char *argv[])
{	unsigned long a1,a2,n,n1,x,y,bat=0,act_nodes;
	unsigned long **narrival,**nblocked,tot_arrival=0,tot_blocked=0,bat_arrival,bat_blocked;
	long **graph=NULL,arg_or_index=-1,arg_nt_index=-1;
	short nopath=0;
	char c1='n';
	double tot_time=0,***b,**av_b,avall_b=0,d1,d2;
	path *pathar;
	unsigned long *pchan;
	time_t sim_time;
	FILE *bp_file=NULL, *maxbp_file=NULL, *minbp_file=NULL, *av_atm_file=NULL, *var_file=NULL, *net_file=NULL;
	char *tstring;

	// Define (FindChannels) as pointer to a function
	short (* FindChannels)(unsigned long y,unsigned long x,unsigned long **pchan, path **ppath,unsigned long req_slots);
	PGetNextInRList = &GetNextInRList;
	PopEvent = &NPopEvent;


	// Randomizing with time
	srand((unsigned) time(NULL));

	tstring = malloc(MAX_FILE_NAME_SIZE*sizeof(*tstring));
	//srand(500);
	rlist1.pr_list = NULL; rlist2.pr_list = NULL;
	//rwa_method=RANDOM_RWA;

	// Getting user options
	for (n=0; n<argc; n++)
		if (argv[n][0]=='-')
			switch (toupper(argv[n][1]))
			{	case 'B':
					bat = atoi(&(argv[n][3]));
				break;
				case 'R':
					if (r_method==NONE) r_method=RANDOM_RWA; else wa_method=RANDOM_RWA;
				break;
				case 'L':
					if (toupper(argv[n][2])=='O')
						local_sh = 1;
					else if (toupper(argv[n][2])=='U')
					{	
						#ifndef USE_PATH_NUM
							printf("\nYou should #define USE_PATH_NUM in the source code to be able to use the -lu\noption.\n");
							exit(0);
						#endif
						if (r_method==NONE) r_method=LU_RWA; else wa_method=LU_RWA;
					}
					else if (toupper(argv[n][2])=='L')
					{	
						#if !defined(USE_LLML_R) || !defined(MIXED_RWA)
							printf("\nYou should #define USE_LLML_R and MIXED_RWA in the source code to be able to use the -ll option.\n");
							exit(0);
						#endif
						if (r_method==NONE) r_method=LL_R; else wa_method=FIRST_FIT;
						/* Note: In paper "Effects of Wavelength Routing and Selection Algorithms on Wavelength Conversion Gain in 
								 WDM Optical Networks" It says that LLR WA reduces to MU for single fiber networks. But in paper: 
								 "A Review of Routing and Wavelength Assignment Approaches for Wavelength- Routed Optical WDM Networks"
								 it says that it reduces to FF WA!!. I'll use FF until they settle it down.
						*/
					}
				break;

				case 'D':
					if (toupper(argv[n][2])=='R') drate = atof(&(argv[n][4]));
				
				break;
				case 'S':
					srand(atoi(&(argv[n][3])));
				break;
				case 'M':
					if (toupper(argv[n][2])=='X')
						max_path_size = atoi(&(argv[n][4]));
					else if (toupper(argv[n][2])=='N')
						min_path_size = atoi(&(argv[n][4]));
					else if (toupper(argv[n][2])=='U')
						{
							#ifndef USE_PATH_NUM
								printf("\nYou should #define USE_PATH_NUM in the source code to be able to use the -mu \noption.\n");
								exit(0);
							#endif
							if (r_method==NONE) r_method=MU_RWA; else wa_method=MU_RWA;
						}
					else if (toupper(argv[n][2])=='L')
					{	
						#if !defined(USE_LLML_R) || !defined(MIXED_RWA)
							printf("\nYou should #define USE_LLML_R and MIXED_RWA in the source code to be able to use the -ml option.\n");
							exit(0);
						#endif
						if (r_method==NONE) r_method=ML_R; else wa_method=FIRST_FIT;
					}
					else if (toupper(argv[n][2])=='P')
						max_paths = atoi(&(argv[n][4]));
				break;
				case 'N':
					if (toupper(argv[n][2])=='T')
						arg_nt_index=n;
				break;
				case 'P':
					if (toupper(argv[n][2])=='X')
						max_swi = atoi(&(argv[n][4]));
					else if (toupper(argv[n][2])=='N')
						min_swi = atoi(&(argv[n][4]));
				break;
				case 'F':
					if (toupper(argv[n][2])=='F') 
						if (r_method==NONE) r_method=FIRST_FIT; else wa_method=FIRST_FIT;
					else 
						factor = atof(&(argv[n][3]));
				break;
				case 'I':
					arg_nt_index = -2;
				break;
				case 'C':
					cofactor = atof(&(argv[n][3]));
				break;
				case 'A':
					if (toupper(argv[n][2])=='T')
						max_attempts = atoi(&(argv[n][4]));
					else
						all_paths = 1;
				break;
				case 'T':
					tot_time = atof(&(argv[n][3]));
				break;
				case 'V':
					if (toupper(argv[n][2])=='I')
						virt_input = 1;
					else if (toupper(argv[n][2])=='O')
						virt_output = 1;
				break;
				case 'O':
					if (toupper(argv[n][2])=='F')
					{	// Naming and preparing output files
						
						strcpy(tstring,&(argv[n][4]));
						strcat(tstring,"_Blocking_Probability.csv");
						bp_file = fopen(tstring,"a+");
						strcpy(tstring,&(argv[n][4]));
						strcat(tstring,"_Max_Blocking_Probability.csv");
						maxbp_file = fopen(tstring,"a+");
						strcpy(tstring,&(argv[n][4]));
						strcat(tstring,"_Min_Blocking_Probability.csv");
						minbp_file = fopen(tstring,"a+");
						strcpy(tstring,&(argv[n][4]));
						strcat(tstring,"_Variance.csv");
						var_file = fopen(tstring,"a+");
						#ifdef CALC_ATTEMPTS
							strcpy(tstring,&(argv[n][4]));
							strcat(tstring,"_Av_No_Attempts.csv");
							av_atm_file = fopen(tstring,"a+");
						#endif

					} else if (toupper(argv[n][2])=='R')
						arg_or_index = (long) n;
				break;
				case '?':
				// Usage is also printed when the user does not provide a network description.
				// i.e. niether the -nt nor the -i option are used
					PrintUsage(1); exit(0);
			}
		/*else 
		{	printf("Invalid Options provided"); exit(0);
		}*/
	free(tstring);
	if (arg_nt_index==-1) { PrintUsage(0); exit(0); }
	SetSeed(rand()+1); SetSeed2(rand()+1);		// +1 because zero seeds will cause trouble

	// Check correctness of RWA 
	CheckParamCorrectness();

	// Open network file if available
	net_file=stdin;
	if (arg_nt_index!=-2) net_file = fopen(&(argv[arg_nt_index][4]),"r");

	// Get Number of nodes
	printf("\nEnter Number of Nodes in The Network: ");
	fscanf(net_file,"%lu",&real_nodes); nodes = real_nodes;

	// Define and Get node capabilities
	ncapab = (node_capabilities *) malloc(nodes*sizeof(*ncapab));
	for (n=0; n<real_nodes; n++)
	{	printf("\nNode %lu capabilities:\n",n+1);
		printf("Choose one of the following node types:\n 1. Non Grooming node\n");
		printf(" 2. Single-Hop Grooming node\n 3. Multi-Hop Full Grooming node\n");
		printf(" 4. Multi-Hop Partial Grooming node\n? ");
		fscanf(net_file,"%hd",&ncapab[n].node_type); 
		ncapab[n].slot_inter = (ncapab[n].node_type==3); 
		if (ncapab[n].node_type==2 || ncapab[n].node_type==4)
			printf("Does the W-Fabric of this node support wavelength conversions (y/n) ? ");
		else printf("Does this node support wavelength conversions (y/n) ? ");
		fscanf(net_file," \n\t%c",&c1); ncapab[n].conversions = (c1=='y');
		
		if (ncapab[n].node_type==2 || ncapab[n].node_type==4)
		{	char s1[] = "ports", s2[] = "cores";
			char *st;
			
			if (ncapab[n].node_type==2)
			{	printf("How many grooming ports are attached to this node ? "); st = s1; }
			else { printf("How many grooming cores (G-Fabric) are attached to this node ? "); st = s2; }
			fscanf(net_file,"%lu",&ncapab[n].attached_nodes.size);
				
			// Creating a virtual node for each grooming port
			a1 = ncapab[n].attached_nodes.size;
			ncapab[n].attached_nodes.ar = (unsigned long *) malloc(a1*sizeof(*(ncapab[n].attached_nodes.ar)));
			ncapab = (node_capabilities *) realloc(ncapab,(nodes+a1)*sizeof(*ncapab));
			
			if (!virt_input)
			{	ncapab[n].attached_incap = (long *) malloc(a1*sizeof(*(ncapab[n].attached_incap)));
				ncapab[n].attached_outcap = (long *) malloc(a1*sizeof(*(ncapab[n].attached_outcap)));

				// Geting link capacities
				printf("Enter INCOMING link capacities on fibers from node %lu to the grooming %s: \n",n+1,st);
				for (n1=0; n1<a1; n1++)
					fscanf(net_file,"%ld",&ncapab[n].attached_incap[n1]);
				printf("Enter OUTGOING link capacities on fibers from the grooming %s to node %lu\n",st,n+1);
				for (n1=0; n1<a1; n1++)
					fscanf(net_file,"%ld",&ncapab[n].attached_outcap[n1]);
				
				printf("Enter the probability that an incoming call will terminate at the grooming %s: ",st);
				fscanf(net_file,"%lg",&ncapab[n].groomin_prob);

				printf("Enter the probability that an outgoing call will start from the grooming %s: ",st);
				fscanf(net_file,"%lg",&ncapab[n].groomout_prob);
			} else
			{	ncapab[n].attached_incap = NULL;
				ncapab[n].attached_outcap= NULL;
			}
			
			// Attaching and printing the nodes
			printf("Virtual node(s):");
			for (n1=0; n1<a1; n1++)
			{	a2=nodes+n1; printf(" %lu",a2);
				ncapab[n].attached_nodes.ar[n1] = a2;
				ncapab[a2].attached_nodes.size = 0;
				ncapab[a2].conversions = 1;  //** Make an option to enter this value
				ncapab[a2].slot_inter = 1;
				ncapab[a2].attached_nodes.ar = NULL;
				ncapab[a2].attached_incap = NULL;
				ncapab[a2].attached_outcap = NULL;
			}
			nodes += a1; printf(" are created and attached to node %lu\n",n+1);
		} else
		{	ncapab[n].attached_nodes.size = 0;
			ncapab[n].attached_nodes.ar = NULL;
			ncapab[n].attached_incap = NULL;
			ncapab[n].attached_outcap = NULL;
		}

	}

	// Allocate arrays
	cap = (long **) malloc(nodes*sizeof(*cap));
	graph = (long **) malloc(nodes*sizeof(*graph));
	load = (double **) malloc(nodes*sizeof(*load));
	//spaths = (path **) malloc(nodes*sizeof(*spaths));
	chstate = (chan_state **) malloc(nodes*sizeof(*chstate));
	narrival = (unsigned long **) malloc(nodes*sizeof(*narrival));
	nblocked = (unsigned long **) malloc(nodes*sizeof(*nblocked));
	minch_cap = (sized_array ***) malloc(nodes*sizeof(*minch_cap));
	z = (unsigned long **) malloc(nodes*sizeof(*z));
	b = (double ***) malloc(nodes*sizeof(*b));
	av_b = (double **) malloc(nodes*sizeof(*av_b));
	if (r_method==MU_RWA || r_method==LU_RWA)
		pr_path_data = (ind_list **) Malloc2D(nodes,nodes,sizeof(**pr_path_data));
	if (wa_method==MU_RWA || wa_method==LU_RWA)
		pr_chan_data.size = 0;
	
	for (n=0; n<nodes; n++)
	{	cap[n] = (long *) calloc((nodes+1),sizeof(**cap));		//+1 for a small trick in findchannels()
		graph[n] = (long *) malloc(nodes*sizeof(**graph));
		load[n] = (double *) malloc(nodes*sizeof(**load));
		//spaths[n] = (path *) malloc(nodes*sizeof(**spaths));
		chstate[n] = (chan_state *) malloc((nodes+1)*sizeof(**chstate));
		narrival[n] = (unsigned long *) calloc(nodes,sizeof(**narrival));
		nblocked[n] = (unsigned long *) calloc(nodes,sizeof(**nblocked));
		minch_cap[n] = (sized_array **) malloc(nodes*sizeof(**minch_cap));
		z[n] = (unsigned long *) calloc(nodes,sizeof(**z));
		b[n] = (double **) malloc(nodes*sizeof(**b));
		av_b[n] = (double *) calloc(nodes,sizeof(**av_b));
	}


	// Get Max Number of Time slots in a wavelength
	printf("Enter Maximum Number of Time Slots in Each Wavelength: ");
	fscanf(net_file,"%lu",&chan_slots);
	printf("Enter Minimum Number of Time Slots a call can consume: ");
	fscanf(net_file,"%lu",&min_call_slots);
	printf("Enter Maximum Number of Time Slots a call can consume: ");
	fscanf(net_file,"%lu",&max_call_slots);
	dif_call_slots = max_call_slots-min_call_slots+1;

	// Seting link capacities for virtual nodes 
	if (!virt_input)
		for (n=0; n<real_nodes; n++)
		{	for (n1=0; n1<ncapab[n].attached_nodes.size; n1++)
			{	cap[ncapab[n].attached_nodes.ar[n1]][n] = ncapab[n].attached_outcap[n1];
				cap[n][ncapab[n].attached_nodes.ar[n1]] = ncapab[n].attached_incap[n1];
			}

			// No more need for the capacities arrays
			free(ncapab[n].attached_outcap); ncapab[n].attached_outcap = NULL;
			free(ncapab[n].attached_incap); ncapab[n].attached_incap = NULL;
		}

	// Get Capacities Matrix and do Other Related Setup
	for (y=0; y<nodes; y++)
	{	
		printf("Enter all capacities on fiber links from node %lu (Ex: 4 0 3 1): ",y+1);
		for (x=0; x<nodes; x++)
		{	
			if (virt_input || (x<real_nodes && y<real_nodes))	
				fscanf(net_file,"%ld",&(cap[y][x]));
			if (x==y) cap[y][x]=0; graph[y][x] = (cap[y][x]!=0);
			chstate[y][x].size = cap[y][x];
			if (chstate[y][x].size>0)
				chstate[y][x].chan = (unsigned long *) calloc(chstate[y][x].size,sizeof(unsigned long));
			else chstate[y][x].chan = NULL;
			chstate[y][x].f_free = 0;
			if (wa_method==MU_RWA || wa_method==LU_RWA)
				pr_chan_data.size = __max(pr_chan_data.size,(unsigned long) cap[y][x]);
			
		}
		cap[y][nodes]=-1;   //**
	}


	// Get Arrival Rates Matrix
	if (virt_input)
		for (y=0; y<nodes; y++)
		{	printf("Enter Arrival rates from node %lu to other nodes: ",y+1);
			for (x=0; x<nodes; x++)
			{	fscanf(net_file,"%lg",&load[y][x]); load[y][x] *= (factor/cofactor);  
				if (x==y) load[y][x]=0; 
			}
		}
	else 
	{	for (y=0; y<nodes; y++)
		{	if (y<real_nodes) printf("Enter Arrival rates from node %lu to other nodes: ",y+1);
			for (x=0; x<nodes; x++)
				if (y>=real_nodes || x>=real_nodes)
					load[y][x]=0;
				else
				{	fscanf(net_file,"%lg",&load[y][x]); load[y][x] *= (factor/cofactor);
					if (x==y) load[y][x]=0; 
				}
		}
		// Distribute a percentage of the load on the grooming ports
		for (y=0; y<real_nodes; y++)
			for (x=0; x<real_nodes; x++)
				if (ncapab[y].attached_nodes.size!=0)
				{	d1 = ncapab[y].groomout_prob * load[y][x] / ncapab[y].attached_nodes.size;
					for (n=0; n<ncapab[y].attached_nodes.size; n++)
						load[ncapab[y].attached_nodes.ar[n]][x] = d1;
					load[y][x] -= ncapab[y].groomout_prob * load[y][x];
				}
		for (y=0; y<nodes; y++)
			for (x=0; x<real_nodes; x++)
				if (ncapab[x].attached_nodes.size!=0)
				{	d1 = ncapab[x].groomin_prob * load[y][x] / ncapab[x].attached_nodes.size;
					for (n=0; n<ncapab[x].attached_nodes.size; n++)
						load[y][ncapab[x].attached_nodes.ar[n]] = d1;
					load[y][x] -= ncapab[x].groomin_prob * load[y][x];
				}
	}
	
	// Read number of batches if -b option is not used
	if (bat==0)
	{	printf("Enter Number of Batches: ");
		fscanf(net_file,"%lu",&bat);
	} else if (net_file!=stdin) fscanf(net_file,"%lu",&a1);

	// Read number total time of each batch if -t is not used
	if (tot_time==0)
	{	printf("Enter Total Time of each batch: ");
		fscanf(net_file,"%lg",&tot_time);
	} else if (net_file!=stdin) fscanf(net_file,"%lg",&d1);

	// Choose Routing method
	a1=0;
	#ifdef MIXED_RWA
		switch (r_method)
		{	case FIRST_FIT:
				ChooseFirstPath = &FFChooseFirstPath; ChoosePath = &FFChoosePath; 
				UpdatePathSuccess = &FFUpdatePathSuccess; break;

			case RANDOM_RWA:
				ChooseFirstPath = &RandChooseFirstPath; ChoosePath = &RandChoosePath; 
				UpdatePathSuccess = &RandUpdatePathSuccess; break;

			case MU_RWA: case LU_RWA:	
				ChooseFirstPath = &LUMUChooseFirstPath; ChoosePath = &LUMUChoosePath; 
				UpdatePathSuccess = &LUMUUpdatePathSuccess;
				PopEvent = &LuMuRPopEvent;
				break;

			case LL_R: case ML_R:
				ChooseFirstPath = &LLMLChooseFirstPath; ChoosePath = &LLMLChoosePath; 
				UpdatePathSuccess = &LLMLUpdatePathSuccess;

				if (r_method==LL_R) CompareLLML = &DesCompareLLML; else CompareLLML = &AscCompareLLML;
				break;
		}
	#endif

	// Choose WA method
	switch (wa_method)
	{	case FIRST_FIT:
			FindChannels = &FFFindChannels; break;
		case RANDOM_RWA:
			FindChannels = &RandFindChannels; break;
		case MU_RWA: case LU_RWA:
			FindChannels = &LUMUFindChannels; 
			if (r_method!=MU_RWA && r_method!=LU_RWA) PopEvent = &LuMuWAPopEvent;
			else PopEvent = &LuMuRWAPopEvent;
			break;
	}

	// When using MU or LU routing, Assign DecreaseRListItemCaller the proper function
	if (r_method==MU_RWA || r_method==LU_RWA)
		DecreaseRListItemCaller = &DecreaseRListItemCallerNormal;

	
	// Choose Least Used or Most Used functions
	if (r_method==MU_RWA) 
	{	DecreaseRListItem = &DecreaseDesListItem; IncreaseRListItem = &IncreaseDesListItem;
	} else 
	if (r_method==LU_RWA) 
	{	DecreaseRListItem = &DecreaseAscListItem; IncreaseRListItem = &IncreaseAscListItem;
	}
	if (wa_method==MU_RWA) 
	{	DecreaseWAListItem = &DecreaseDesListItem; IncreaseWAListItem = &IncreaseDesListItem; }
	else if (wa_method==LU_RWA) 
	{	DecreaseWAListItem = &DecreaseAscListItem; IncreaseWAListItem = &IncreaseAscListItem; }



	// Randomize seeds and initialize blocking probabilities
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++) 
		{	z[y][x]=(unsigned) rand()+1;
			b[y][x]=(double *) malloc(bat*sizeof(***b));
		}

	// Find Required (or Shortest) Paths
	{	long **pshortest,**pshortest2;
		
		if (all_paths)
		{	gpaths = FindGraphPathsExt(graph,nodes,NEG_INFINITY,INFINITY,min_path_size,max_path_size);
			//TruncateGraphPaths(gpaths,nodes,1);
		} else
		{	
			pshortest = FindGraphLeastCosts(graph,nodes);
			pshortest2 = (long **) Malloc2D(nodes,nodes,sizeof(**pshortest2));
			Copy2D(pshortest2,pshortest,nodes,nodes);


			MaxConst2D(pshortest,nodes,nodes,(long) min_path_size);
			MinConst2D(pshortest2,nodes,nodes,(long) max_path_size);
			gpaths = FindGraphPathsExt2(graph,nodes,NULL,NULL,pshortest,pshortest2);

			if (max_paths!=0) TruncateGraphPaths(gpaths,nodes,max_paths);

			Free2DArray(pshortest,nodes); Free2DArray(pshortest2,nodes);

			//TruncateGraphPaths(gpaths,nodes,1);
			//printf("Selected Shortest Paths:\n\n");
		} 

		/* gpaths = CreateEmptyGraphPaths(nodes);
		printf("\n");
		for (y=0; y<nodes; y++)
		{	pathar=DijkstraPaths(graph,nodes,y);
			for (x=0; x<nodes; x++)
			{	if (y==x) { gpaths[y][x].size=0; continue; }
				gpaths[y][x].ppaths = malloc(sizeof(path));
				gpaths[y][x].ppaths[0] = pathar[x];
				gpaths[y][x].size = ((pathar[x].par==NULL) ? 0:1);
				//PrintPath(y,x,pathar[x]);
			}
			//printf("----------------------------------------------\n");
			free(pathar);
		} */

		// Checking if there are no paths between any source destination pair that has load
		for (y=0; y<nodes; y++)
			for (x=0; x<nodes; x++)
				if (!gpaths[y][x].size && load[y][x]!=0)
				{	printf("Warning: There are no paths from %lu to %lu\n",y+1,x+1);
					nopath=1;
				}
	}

	// Adding cycle paths for partial grooming nodes
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
		{	a1=gpaths[y][x].size;
			for (n=0; n<a1; n++)
				AddPartialNodesPaths(y,x,n);
			for (n=a1; n<gpaths[y][x].size; n++)
				gpaths[y][x].ppaths[n].tot_cost = CalcPathCost(y,graph,gpaths[y][x].ppaths[n]);
		}

	PrintGraphPaths(gpaths,nodes);



	// Finding Minimum chstate capacities for each y to x in each path
	AllocateMinCapacitiesArrays(gpaths,nodes);
	for (y=0; y<nodes; y++)
	{	for (x=0; x<nodes; x++)	
			for (n=0; n<gpaths[y][x].size; n++)
				if (x==y) 
				{	minch_cap[y][x][n].ar=NULL; continue; 
				}
				else 
					GetPathMinCapacities(y,&(gpaths[y][x].ppaths[n]),&(minch_cap[y][x][n]));
	}


	// "Most Used" WA or "Least Used" WA Case
	if (wa_method==MU_RWA || wa_method==LU_RWA)
		// Allocating global channels list
		AllocateIndexedList(&pr_chan_data,pr_chan_data.size);

	
	// "Most Used" Routing or "Least Used" Routing Case
	if (r_method==MU_RWA || r_method==LU_RWA)
		// Allocating paths "Most Used" or "Least Used" list for each source to destination pair
		for (y=0; y<nodes; y++)
			for (x=0; x<nodes; x++)
				AllocateIndexedList(&(pr_path_data[y][x]),gpaths[y][x].size);



	#ifndef BLOCK_IF_NOPATH
		if (nopath) 
		{	printf("Can not continue simulation. define BLOCK_IF_NOPATH directive in the program to deal will this case");
			goto ExitProg;
		}
	#endif
	
	// Start Simulation
	sim_time = time(NULL);
	printf("\nSimulation Started....");
	for (n1=0; n1<bat; n1++)
	{	path *ppath;
		
		printf("\nBatch %lu:\n",n1+1);
		cu_time=0; fr=NULL;
		Init_chstate();

		if (wa_method==MU_RWA || wa_method==LU_RWA) InitializeIndexedList(&pr_chan_data);
		if (r_method==MU_RWA || r_method==LU_RWA)
			for (y=0; y<nodes; y++)
				for (x=0; x<nodes; x++)
					InitializeIndexedList(&(pr_path_data[y][x]));
				

		// Reset #arrival and #blocked counts
		for (n=0; n<nodes; n++)
		{	memset(narrival[n],0,nodes*sizeof(**narrival));
			memset(nblocked[n],0,nodes*sizeof(**nblocked));
		}
		bat_arrival=0; bat_blocked=0;
		
		// Initialize the events linked list with arrivals
		for (y=0; y<nodes; y++)
			for (x=0; x<nodes; x++)
			{	if (load[y][x]) GenArrival(y,x);
			}
		
		// Inner Main loop
		while (cu_time<tot_time)
		{	unsigned long from,to,nslots;
			unsigned long first_from=0;
			
			from = fr->from; to = fr->to;
			if (IS_ARRIVAL())
			{	cu_time = fr->t; nslots = fr->nslots;
				PopEvent(&fr);
				pchan=NULL;
				if (FindChannels(from,to,&pchan,&ppath,nslots))
				{
					GenDeparture(from,to,pchan,ppath,nslots);

					// Reserve Channels
					ReserveChannels(from,to,pchan,ppath,nslots);
				}
				else { nblocked[ARRIVAL_FROM()][to]++; }

				// Increase #of arrivals then generate the next arrival
				narrival[from][to]++; GenArrival(from,to);

			
			} else //if ARRIVAL
			{	
				// If Departure, Free reserved Channels
				FreeChannels(from,to,fr->pchan,fr->ppath,fr->nslots);

				/* if (rwa_method==MU_RWA)
				{	DecreaseListItem(&(pr_path_data[from][to]),&(pr_path_data[from][to].plist[fr->path_num]),fr->nslots);
					for (n=0; n<fr->ppath->size; n++)
						DecreaseListItem(&pr_chan_data,&(pr_chan_data.plist[fr->pchan[n]]),fr->nslots);
				} */
				PopEvent(&fr);
			}

		}

		// Calculating Blocking Rates
		if (virt_output)
			for (y=0; y<nodes; y++)
				for (x=0; x<nodes; x++) 
				{	if (x==y) continue;
					if (narrival[y][x])
					{	b[y][x][n1] = (double) nblocked[y][x]/narrival[y][x];
						bat_arrival += narrival[y][x];
						bat_blocked += nblocked[y][x];
						av_b[y][x] = av_b[y][x]+b[y][x][n1];
						printf("\nbp[%lu][%lu] = %-5g	 Arrived: %lu	Blocked: %lu",y+1,x+1,b[y][x][n1],narrival[y][x],nblocked[y][x]);
					}
				}
		else
			for (y=0; y<real_nodes; y++)
				for (x=0; x<real_nodes; x++) 
				{	//if (x==y) continue;
					
					// Accumulating virtual nodes arrivals and blocked counts into the thier real nodes
					for (a1=0; a1<ncapab[y].attached_nodes.size; a1++)
					{	narrival[y][x] += narrival[ncapab[y].attached_nodes.ar[a1]][x];
						nblocked[y][x] += nblocked[ncapab[y].attached_nodes.ar[a1]][x];
						for (a2=0; a2<ncapab[x].attached_nodes.size; a2++)
						{	narrival[y][x] += narrival[ncapab[y].attached_nodes.ar[a1]][ncapab[x].attached_nodes.ar[a2]];
							nblocked[y][x] += nblocked[ncapab[y].attached_nodes.ar[a1]][ncapab[x].attached_nodes.ar[a2]];
						}
					}
					for (a2=0; a2<ncapab[x].attached_nodes.size; a2++)
					{	narrival[y][x] += narrival[y][ncapab[x].attached_nodes.ar[a2]];
						nblocked[y][x] += nblocked[y][ncapab[x].attached_nodes.ar[a2]];
					}

					if (narrival[y][x])
					{	b[y][x][n1] = (double) nblocked[y][x]/narrival[y][x];
						bat_arrival += narrival[y][x];
						bat_blocked += nblocked[y][x];
						av_b[y][x] = av_b[y][x]+b[y][x][n1];
						printf("\nbp[%lu][%lu] = %-5g	 Arrived: %lu	Blocked: %lu",y+1,x+1,b[y][x][n1],narrival[y][x],nblocked[y][x]);
					}
				}
		printf("\n----------------------------------------------");
		tot_arrival += bat_arrival; tot_blocked += bat_blocked;
		avall_b += (double) bat_blocked / bat_arrival;
	
		// Free the list
		while (fr!=NULL) NPopEvent(&fr);
		//if (fr->etype==DEPARTURE) FreeChannels(fr->from,fr->to,fr->pchan,fr->ppath,fr->nslots); 
	}


	// Average blocking rates and confidance intervals
	if (virt_output) act_nodes = nodes; else act_nodes = real_nodes;
	printf("\n\nAverage Blocking Rates:\n\n");
	for (y=0; y<act_nodes; y++)
	{	//double a1;
		for (x=0; x<act_nodes; x++) 
		{	if (!local_sh && x==y) continue;
			av_b[y][x] = av_b[y][x]/bat;
			printf("Av.bp[%lu][%lu] = %g\n",y+1,x+1,av_b[y][x]);
	
			//a1=Var(b[y][x],bat,av_b[y][x]); 
			//printf("Variance: %g\n",a1); 
			//a1=(double) td*sqrt(a1/bat);
			//printf("Confidence Interval: %g +/- %g\n",av_b[y][x],a1);
			//printf("Interval Limits: %g to %g\n\n",av_b[y][x]-a1,av_b[y][x]+a1);
		}
	}


	// ------------------------------------------------------------------------------
	// Writing Final Results
	sim_time=time(NULL)-sim_time; avall_b = avall_b / bat;
	d2 = Average2D(av_b,act_nodes,local_sh);
	
	// Writing Network Blocking Probabilities
	printf("\nTotal Blocked = %lu",tot_blocked);
	printf("\nTotal Arrived = %lu",tot_arrival);
	printf("\nAverage Blocking Probability = %g",avall_b);
	printf("\nOverall Blocking Probability = %g",(double) tot_blocked/tot_arrival);
	printf("\nAverage of Averages Blocking Probability = %g",d2);
	#if BP_METHOD==OVERALL_BP
		PrintValueToCSV(bp_file,(double) tot_blocked/tot_arrival,arg_or_index,argv);
	#elif BP_METHOD==AOA_BP
		PrintValueToCSV(bp_file,d2,arg_or_index,argv);
	#else 
		PrintValueToCSV(bp_file,avall_b,arg_or_index,argv);
	#endif


	// Maximum blocking probability
	d1=Max2D(av_b,act_nodes,&y,&x,local_sh);
	printf("\nMaximum Blocking Probability is Av.bp[%lu][%lu] = %g",y+1,x+1,d1);
	PrintValueToCSV(maxbp_file,d1,arg_or_index,argv);

	// Minimum blocking probability
	d1=Min2D(av_b,act_nodes,&y,&x,local_sh);
	printf("\nMinimum Blocking Probability is Av.bp[%lu][%lu] = %g",y+1,x+1,d1);
	PrintValueToCSV(minbp_file,d1,arg_or_index,argv);

	// Variance
	#if BP_METHOD==OVERALL_BP
		d1=Var2D(av_b,act_nodes,(double) tot_blocked/tot_arrival,local_sh);
	#elif BP_METHOD==AOA_BP
		d1=Var2D(av_b,act_nodes,d2,local_sh);
	#else
		d1=Var2D(av_b,act_nodes,avall_b,local_sh);
	#endif
	printf("\nVariance = %g",d1);
	PrintValueToCSV(var_file,d1,arg_or_index,argv);

	// Average No. Attempts
	#ifdef CALC_ATTEMPTS
		printf("\nAverage Number of attempts = %g",(double) tot_attempts/tot_arrival);
		PrintValueToCSV(av_atm_file,(double) tot_attempts/tot_arrival,arg_or_index,argv);
	#endif

	printf("\nSimulation Time: %ld sec\n",sim_time);
	
	
	//------------------------------------------------------------------------------
	// Cleaning
ExitProg:
	Free2DArray(cap,nodes); Free2DArray(graph,nodes); Free2DArray(load,nodes); 
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++) 
		{	free(chstate[y][x].chan);

			for (n=0; n<gpaths[y][x].size; n++)
				free(minch_cap[y][x][n].ar);
			if (r_method==MU_RWA || r_method==LU_RWA)
				FreeIndexedList(&(pr_path_data[y][x]));
		}

	Free2DArray(chstate,nodes); Free3DArray(minch_cap,nodes,nodes);
	Free2DArray(narrival,nodes); Free2DArray(nblocked,nodes); 
	Free2DArray(z,nodes); Free3DArray(b,nodes,nodes); Free2DArray(av_b,nodes);
	Free2DArray(pr_path_data,nodes); 
	free(rlist1.pr_list); free(rlist2.pr_list);

	for (n=0; n<nodes; n++)
	{	free(ncapab[n].attached_nodes.ar);
		free(ncapab[n].attached_incap);
		free(ncapab[n].attached_outcap);
	}
	free(ncapab); 
	FreeGraphPaths(gpaths,nodes);
	if (wa_method==MU_RWA || wa_method==LU_RWA)
		FreeIndexedList(&(pr_chan_data));
	#if defined(USE_LLML_R) && defined(MIXED_RWA)
		free(ptemp2); free(ptemp3);
	#endif
	fclose(net_file);

	// And with a :} I end my ugly code.
}