#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include "NDarrays.h"


void **Malloc2D(unsigned long d1_size,unsigned long d2_size,unsigned long elem_bytes)
{	void ** ar2D;
	unsigned long n;

	if (d1_size>0) 
	{	ar2D = malloc(d1_size*sizeof(void *));
		if (d2_size>0)
			for (n=0; n<d1_size; n++)
				ar2D[n] = malloc(d2_size*elem_bytes);
		else 
			for (n=0; n<d1_size; n++)
				ar2D[n] = NULL;
	}
	else ar2D=NULL;

	return ar2D;
}

void Copy2D(long **d_ar2D,const long **s_ar2D,unsigned long d1_size,unsigned long d2_size)
{	// Copies the constents of a 2D array from s_ar2D to d_ar2D
	unsigned long y;
	
	for (y=0; y<d1_size; y++)
		memcpy(d_ar2D[y],s_ar2D[y],sizeof(**d_ar2D)*d2_size);
}

void Print2D(long **ar2D,unsigned long d1_size,unsigned long d2_size)
{	unsigned long x,y;		
	
	for (y=0; y<d1_size; y++)
		{	for (x=0; x<d2_size; x++)
			{	printf("%ld ",ar2D[y][x]);
			}
			printf("\n");
		}
}

/* void Print2D(double **ar2D,unsigned long d1_size,unsigned long d2_size)
{	unsigned long x,y;		
	
	for (y=0; y<d1_size; y++)
		{	for (x=0; x<d2_size; x++)
			{	printf("%lg ",ar2D[y][x]);
			}
			printf("\n");
		}
} */

void AddConst2D(long **ar2D,unsigned long d1_size,unsigned long d2_size,long c)
{	// Simply adds a constant to a 2D array
	unsigned long y,x;

	for (y=0; y<d1_size; y++)
		for (x=0; x<d2_size; x++)
			ar2D[y][x] += c;
}

void SetConst2D(long **ar2D,unsigned long d1_size,unsigned long d2_size,long c)
{	// Simply adds a constant to a 2D array
	unsigned long y,x;

	for (y=0; y<d1_size; y++)
		for (x=0; x<d2_size; x++)
			ar2D[y][x] = c;
}

void MinConst2D(long **ar2D,unsigned long d1_size,unsigned long d2_size,long c)
{	
	unsigned long y,x;

	for (y=0; y<d1_size; y++)
		for (x=0; x<d2_size; x++)
			if (ar2D[y][x]>c) ar2D[y][x]=c;
}

void MaxConst2D(long **ar2D,unsigned long d1_size,unsigned long d2_size,long c)
{
	unsigned long y,x;

	for (y=0; y<d1_size; y++)
		for (x=0; x<d2_size; x++)
			if (ar2D[y][x]<c) ar2D[y][x]=c;
}

void Free2DArray(void ** ar,unsigned long size_d1)
{	unsigned long n;
	
	if (ar!=NULL)
	{	for (n=0; n<size_d1; n++)
			free(ar[n]);
		free(ar);
	}
}

void Free3DArray(void *** ar,unsigned long size_d1,unsigned long size_d2)
{	unsigned long n;
	
	if (ar!=NULL)
	{	for (n=0; n<size_d1; n++)
			Free2DArray(ar[n],size_d2);
		free(ar);
	}
}

void InitializeDynBuffer(dyn_buffer *pdbuf,unsigned long init_x_size)
{	pdbuf->init_x_size = init_x_size;
	pdbuf->x = 0;
	pdbuf->y_size = 1;
	pdbuf->pbuf = malloc(sizeof(*(pdbuf->pbuf)));
	pdbuf->pbuf[0] = malloc(init_x_size*sizeof(unsigned char));
}
 
void *DynAlloc(dyn_buffer *pdbuf,unsigned long bsize)
{	// Memory pointers returned by this function can not be freed by free(); 
	// You have to use Free2DArray to free the whole pdbuf->pbuf buffer
	void *p1;
	unsigned long a1;

	if (pdbuf->x+bsize>pdbuf->init_x_size)
	{	// Try to shrink the current block to save space
		p1 = _expand(pdbuf->pbuf[pdbuf->y_size-1],pdbuf->x);
		if (p1!=NULL) pdbuf->pbuf[pdbuf->y_size-1]=p1;
		
		// Allocate a new block
		if (pdbuf->x!=0)
		{	pdbuf->y_size++;
			pdbuf->pbuf = realloc(pdbuf->pbuf,pdbuf->y_size*sizeof(*(pdbuf->pbuf)));
		} else free(pdbuf->pbuf[pdbuf->y_size-1]);
		pdbuf->pbuf[pdbuf->y_size-1] = malloc(__max(pdbuf->init_x_size,bsize)*sizeof(unsigned char));
		pdbuf->x = 0;
		a1 = 0; pdbuf->x = bsize;
	} else 
	{ a1 = pdbuf->x; pdbuf->x += bsize; }

	return &(pdbuf->pbuf[pdbuf->y_size-1][a1]);
}

void FreeDynBuffer(dyn_buffer *pdbuf)
{	
	Free2DArray(pdbuf->pbuf,pdbuf->y_size);
}

