// A tiny library of graph manipulation functions writen for Hegons Simulator

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "graphs.h"


void PrintPath(unsigned long y, unsigned long x, path pr_path)
{	unsigned long n1;

	printf("\nPath From %lu to %lu: %lu",y+1,x+1,y+1);
	/* do
	{	printf("-%lu",pr_path.par[n1]+1);
		n1++;
	} while (pr_path.par[n1-1]!=x);
	printf("\nCost: %ld\n",pr_path.tot_cost); */

	for (n1=0; n1<pr_path.size; n1++)
		printf("-%lu",pr_path.par[n1]+1);
	printf("\nCost: %ld\n",pr_path.tot_cost);
}

void PrintGraphPaths(sized_paths **graph_paths,unsigned long nodes)
{	unsigned long y,x,n1;

	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++) 
			for (n1=0; n1<graph_paths[y][x].size; n1++) 
				PrintPath(y,x,graph_paths[y][x].ppaths[n1]);
}

long **FindGraphLeastCosts(long **graph,unsigned long nodes)
{	
	long **lcosts;
	unsigned long n;

	lcosts = malloc(nodes*sizeof(*lcosts));
	for (n=0; n<nodes; n++)
	{	lcosts[n] = Dijkstra(graph,nodes,n);
	}
	return lcosts;
}


long *Dijkstra(long **graph,unsigned long nodes,unsigned long startn)
{	/*	Returns the shortest costs to go from node (startn) to all other nodes
		Requires Constant INFINITY to be defined
		Uses path, and struct dij_elm
	*/
	
	unsigned long n,last,min_i,a2;
	long min,a1;
	dij_elm *q,temp;
	unsigned long *qi;
	long *pshortest;


	// Prepare list
	last = nodes-1;
	pshortest = (long *) malloc(nodes*sizeof(*pshortest));
	q = (dij_elm *) malloc(nodes*sizeof(*q));
	qi = (unsigned long *) malloc(nodes*sizeof(*qi));

	for (n=0; n<nodes; n++) 
	{	q[n].dist=INFINITY; qi[n]=n; q[n].node=n;
	}
	
	// Distance from the first node to itself is always 0
	q[startn].node=last; qi[last]=startn;
	q[last].node=startn; q[last].dist=0;
	qi[startn]=last; min=0; min_i=startn; pshortest[min_i]=min;

	while (last>0)
	{	last--;

		// Relax All edges reachable
		for (n=0; n<nodes; n++)
		{	if (graph[min_i][n]!=0)
			{	a1 = graph[min_i][n]+min;
				if (a1<q[qi[n]].dist || q[qi[n]].dist==INFINITY) 
					q[qi[n]].dist=a1;
			}
		}

		// Find Node with minimum distance in the q. And create the path for it
		min=INFINITY;
		for (n=0; n<=last; n++)
		{	if (q[n].dist<min) { min=q[n].dist; min_i=q[n].node; }
		}

		// If a path is available.
		if (min!=INFINITY) 
			pshortest[min_i]=min;
		else 
		{	// Otherwise mark distances as INFINITY and exit.
			for (n=0; n<=last; n++)
				pshortest[q[n].node]=INFINITY;
			break;
		}

		// Removing the node from the q by swaping it with the last one.
		temp=q[qi[min_i]]; q[qi[min_i]]=q[last]; q[last]=temp;
		a2=qi[min_i]; qi[min_i]=last; qi[q[a2].node]=a2;
	}


	// Cleaning
	free(q); free(qi); 

	return pshortest;
}

/*long *NthDijkstra(long **graph,unsigned long nodes,unsigned long startn,long *prev_pshortest)
{	/*	Returns the shortest costs to go from node (startn) to all other nodes
		such that the results must be more than the previous shortest results given by prev_pshortest
		Requires Constant INFINITY to be defined
		Uses path, and struct dij_elm

		This function is used to find the nth shortest paths.
	
	
	unsigned long n,last,flast,min_i,a2;
	long min,a1;
	dij_elm *q,temp;
	unsigned long *qi,fq;					// fq: found nodes queue
	long *pshortest;


	// Prepare list
	last = nodes-1; flast = last;
	pshortest = (long *) malloc(nodes*sizeof(*pshortest));
	q = (dij_elm *) malloc(nodes*sizeof(*q));
	qi = (unsigned long *) malloc(nodes*sizeof(*qi));
	fq = (unsigned long *) malloc(nodes*sizeof(*fq));

	for (n=0; n<nodes; n++) 
	{	q[n].dist=INFINITY; qi[n]=n; q[n].node=n;
		fq[n]=n; pshortest[n]=INFINITY;
	}
	
	// Distance from the first node to itself is always 0
	q[startn].node=last; qi[last]=startn;
	q[last].node=startn; q[last].dist=0;
	qi[startn]=last; min=0; min_i=startn; pshortest[min_i]=min;
	
	// We are not looking for nth ordered paths from a node to it self, so we exclude the node
	// it self
	fq[startn]=flast; fq[flast]=startn; flast--;

	while (last>0)
	{	last--;

		// Relax All edges reachable
		for (n=0; n<nodes; n++)
		{	if (graph[min_i][n]!=0)
			{	a1 = graph[min_i][n]+min;
				if (a1<q[qi[n]].dist || q[qi[n]].dist==INFINITY) 
					q[qi[n]].dist=a1;
			}
		}


		// Begin nth paths: ------------------	
		// Find node with minimum distance in the q such that it is more than the previous distance
		min=INFINITY;
		for (n=0; n<=flast; n++)
		{	a1 = q[qi[fq[n]]].dist;
			if (a1<min && a1>prev_pshortest[fq[n]]) { min=a1; min_i=n; }
		}		

		// If an nth path is available. store the resultant hop count and remove by swaping with flast
		if (min!=INFINITY) 
		{	unsigned long a2;

			pshortest[fq[min_i]]=min; a2=fq[min_i]; fq[min_i]=fq[flast]; fq[flast]=a2;
			flast--;
		}
		// End of nth paths ------------------


		// Find Node with minimum distance in the q. And create the path for it
		min=INFINITY;
		for (n=0; n<=last; n++)
			if (q[n].dist<min) { min=q[n].dist; min_i=q[n].node; }
		

		// If a path is not available.
		if (min==INFINITY) 
		{	// Mark distances as INFINITY and exit.
			for (n=0; n<=last; n++)
				pshortest[q[n].node]=INFINITY;
			break;
		}

		// Removing the node from the q by swaping it with the last one.
		temp=q[qi[min_i]]; q[qi[min_i]]=q[last]; q[last]=temp;
		a2=qi[min_i]; qi[min_i]=last; qi[q[a2].node]=a2;
	}


	// Cleaning
	free(q); free(qi); 

	return pshortest;
} */

path *DijkstraPaths(long **graph,unsigned long nodes,unsigned long startn)
{	/*	This is similar to Dijkstra function but returns the actual shortest paths found.
		Unfortunatly, If multiple shrotest paths are found from a start node to a destination
		this function will return only one of the paths. More code and modifications are needed
		to return all shortest paths.
		
		startn is the first node in path. (starting Node)
		Requires Constant INFINITY to be defined
		Uses path, and struct dij_elm
		If the EXTRA constant is =1 this code will allocate an extra element after each path
		and assigne it the <nodes> variable. This is to do a small trick later.
		If this function is to be used outside this program. You should #define EXTRA 0
	*/
	
	unsigned long n,last,min_i,a2;
	long min,a1;
	dij_elm *q,temp;
	unsigned long *qi;
	path *paths;


	// Prepare list
	paths = (path *) malloc(nodes*sizeof(*paths));
	q = (dij_elm *) malloc(nodes*sizeof(*q));
	last = nodes-1;
	qi = (unsigned long *) malloc(nodes*sizeof(*qi));

	for (n=0; n<nodes; n++) 
	{	q[n].dist=INFINITY; qi[n]=n; q[n].node=n;
		paths[n].par=NULL; paths[n].size=0;
	}
	q[startn].node=last; qi[last]=startn;
	q[last].node=startn; q[last].dist=0;
	qi[startn]=last; min=0; min_i=startn;


	while (last>0)
	{	last--;

		// Relax All edges reachable
		for (n=0; n<nodes; n++)
			if (graph[min_i][n]!=0)
			{	a1 = graph[min_i][n]+min;
					if (a1<q[qi[n]].dist || q[qi[n]].dist==INFINITY) 
				{	q[qi[n]].dist=a1;
						q[qi[n]].prnode=min_i;
				}
			}
		

		// Find Node with minimum distance in the q. And create the path for it
		min=INFINITY;
		for (n=0; n<=last; n++)
		{	if (q[n].dist<min) { min=q[n].dist; min_i=q[n].node; }
		}
		
		// If a path is available.
		if (min!=INFINITY) 
		{	paths[min_i].tot_cost=min;
			a2=q[qi[min_i]].prnode; paths[min_i].size = paths[a2].size+1;
			paths[min_i].par = (unsigned long *) malloc((paths[min_i].size+EXTRA)*sizeof(*(paths->par)));
			memcpy(paths[min_i].par,paths[a2].par,(paths[min_i].size-1)*sizeof(*(paths->par)));
			paths[min_i].par[paths[min_i].size-1] = min_i;
			#if EXTRA==1
				paths[min_i].par[paths[min_i].size] = nodes;
			#endif
		}
		else 
		// Otherwise exit.
			break;


		// Removing the node from the q by swaping it with the last one.
		temp=q[qi[min_i]]; q[qi[min_i]]=q[last]; q[last]=temp;
		a2=qi[min_i]; qi[min_i]=last; qi[q[a2].node]=a2;
	}

	// Cleaning
	free(q); free(qi); 

	return paths;
}


// -------------------------------------------------------------------------------
long max_cost=INFINITY,min_cost=NEG_INFINITY;
unsigned long max_size=UINFINITY,min_size=0;

void FindPathsExt(long **graph, unsigned long nodes,long fnode, sized_paths *paths_store,long lmin_cost,long lmax_cost ,unsigned long lmin_size,unsigned long lmax_size)
{	
	max_cost = lmax_cost;
	min_cost = lmin_cost;
	max_size = lmax_size;
	min_size = lmin_size;
	FindPaths(graph,nodes,fnode,paths_store);

	max_cost=INFINITY; min_cost=NEG_INFINITY;
	max_size=UINFINITY; min_size=0;

}

void FindPaths(long **graph, unsigned long nodes,long fnode, sized_paths *paths_store)
{	/*	If the EXTRA constant is =1 this code will allocate an extra element after each path
		and assigne it the <nodes> variable. This is to do a small trick later.
		If this function is to be used outside this program. You should #define EXTRA 0
	*/

	static short *visited = NULL;
	static path cu_path;
	static long root_node=-1;
	static unsigned long a1;
	unsigned long n;


	// Initialize if it's the root node
	if (root_node==-1)
	{	root_node = fnode;
		visited = (short *) calloc(nodes,sizeof(*visited));
		cu_path.par = NULL; cu_path.tot_cost = 0; cu_path.size = 0;
	}
	
	visited[fnode]=1;
	for (n=0; n<nodes; n++)
		if (!visited[n] && graph[fnode][n]>0)
		{	// Add newly found link to current path
			cu_path.size++;
			cu_path.par = realloc(cu_path.par,cu_path.size*sizeof(*(cu_path.par)));
			cu_path.par[cu_path.size-1] = n; cu_path.tot_cost += graph[fnode][n];
			
			// Create a new path and copy current path to it.
			if (cu_path.size>=min_size && cu_path.tot_cost>=min_cost && cu_path.tot_cost<=max_cost)
			{	a1 = (paths_store[n].size++);
				paths_store[n].ppaths = realloc(paths_store[n].ppaths,paths_store[n].size*sizeof(*(paths_store[n].ppaths)));
				paths_store[n].ppaths[a1].size = cu_path.size;
				paths_store[n].ppaths[a1].tot_cost = cu_path.tot_cost;
				paths_store[n].ppaths[a1].par = malloc((cu_path.size+EXTRA)*sizeof(*(cu_path.par)));
				memcpy(paths_store[n].ppaths[a1].par,cu_path.par,cu_path.size*sizeof(*(cu_path.par)));
				#if EXTRA==1
					paths_store[n].ppaths[a1].par[cu_path.size] = nodes;
				#endif
			}

			// visit the new node
			if (cu_path.size<max_size)
				FindPaths(graph,nodes,n,paths_store);
			
			// Deallocate the last found link from current path
			cu_path.size--;
			cu_path.par = realloc(cu_path.par,cu_path.size*sizeof(*(cu_path.par)));
			cu_path.tot_cost -= graph[fnode][n];
		}


	// Free temporary memory if were out of the root node
	if (fnode==root_node)
	{	root_node = -1;
		free(visited); free(cu_path.par);
	} 
	else visited[fnode]=0;
}


void FindPathsExt2(long **graph, unsigned long nodes,long fnode, sized_paths *paths_store, long *pmin_cost, long *pmax_cost, unsigned long *pmin_size, unsigned long *pmax_size)
{	/*	If the EXTRA constant is =1 this code will allocate an extra element after each path
		and assigne it the <nodes> variable. This is to do a small trick later.
		If this function is to be used outside this program. You should #define EXTRA 0
	*/

	static short *visited = NULL;
	static path cu_path;
	static long root_node=-1;
	static unsigned long a1,max_of_max_size;
	unsigned long n;


	// Initialize if it's the root node
	if (root_node==-1)
	{	root_node = fnode;
		visited = (short *) calloc(nodes,sizeof(*visited));
		cu_path.par = NULL; cu_path.tot_cost = 0; cu_path.size = 0;
		
		// Calculate the maximum path size required to avoid searching deeper
		max_of_max_size=0;
		for (n=0; n<nodes; n++)
			max_of_max_size = __max(pmax_size[n],max_of_max_size);
	}
	
	visited[fnode]=1;
	for (n=0; n<nodes; n++)
		if (!visited[n] && graph[fnode][n]>0)
		{	// Add newly found link to current path
			cu_path.size++;
			cu_path.par = realloc(cu_path.par,cu_path.size*sizeof(*(cu_path.par)));
			cu_path.par[cu_path.size-1] = n; cu_path.tot_cost += graph[fnode][n];
			
			// Create a new path and copy current path to it.
			if (AVOIDNULL(pmin_size,cu_path.size>=pmin_size[n]) && AVOIDNULL(pmin_cost,cu_path.tot_cost>=pmin_cost[n]) && AVOIDNULL(pmax_size,cu_path.size<=pmax_size[n]) && AVOIDNULL(pmax_cost,cu_path.tot_cost<=pmax_cost[n]))
			{	a1 = (paths_store[n].size++);
				paths_store[n].ppaths = realloc(paths_store[n].ppaths,paths_store[n].size*sizeof(*(paths_store[n].ppaths)));
				paths_store[n].ppaths[a1].size = cu_path.size;
				paths_store[n].ppaths[a1].tot_cost = cu_path.tot_cost;
				paths_store[n].ppaths[a1].par = malloc((cu_path.size+EXTRA)*sizeof(*(cu_path.par)));
				memcpy(paths_store[n].ppaths[a1].par,cu_path.par,cu_path.size*sizeof(*(cu_path.par)));
				#if EXTRA==1
					paths_store[n].ppaths[a1].par[cu_path.size] = nodes;
				#endif
			}

			// Visit the new node
			if (cu_path.size<max_of_max_size)
				FindPathsExt2(graph,nodes,n,paths_store,pmin_cost,pmax_cost,pmin_size,pmax_size);
			
			// Deallocate the last found link from current path
			cu_path.size--;
			cu_path.par = realloc(cu_path.par,cu_path.size*sizeof(*(cu_path.par)));
			cu_path.tot_cost -= graph[fnode][n];
		}


	// Free temporary memory if were out of the root node
	if (fnode==root_node)
	{	root_node = -1;
		free(visited); free(cu_path.par);
	} 
	else visited[fnode]=0;
}


sized_paths **FindGraphPaths(long **graph,unsigned long nodes)
{	// Finds All possible non-cyclic graph paths
		
	sized_paths **graph_paths;

	graph_paths = FindGraphPathsExt(graph,nodes,min_cost,max_cost,min_size,max_size);
	return graph_paths;
}


sized_paths **FindGraphPathsExt(long **graph,unsigned long nodes,long lmin_cost,long lmax_cost ,unsigned long lmin_size,unsigned long lmax_size)
{	// Finds All possible non-cyclic graph paths with extra options

	sized_paths **graph_paths;
	unsigned long y;

	graph_paths = CreateEmptyGraphPaths(nodes);
	for (y=0; y<nodes; y++)	
		FindPathsExt(graph,nodes,y,graph_paths[y],lmin_cost,lmax_cost,lmin_size,lmax_size);
	return graph_paths;
}

sized_paths **FindGraphPathsExt2(long **graph,unsigned long nodes,long **pgraph_min_cost, long **pgraph_max_cost, unsigned long **pgraph_min_size, unsigned long **pgraph_max_size)
{	// Finds All possible non-cyclic graph paths with extra options

	sized_paths **graph_paths;
	unsigned long y;

	graph_paths = CreateEmptyGraphPaths(nodes);
	for (y=0; y<nodes; y++)
		FindPathsExt2(graph,nodes,y,ARNULL(graph_paths,y),ARNULL(pgraph_min_cost,y),ARNULL(pgraph_max_cost,y),ARNULL(pgraph_min_size,y),ARNULL(pgraph_max_size,y));
	return graph_paths;
}

void TruncateSizedPaths(sized_paths *p_paths, unsigned long max_paths)
{	
	unsigned long n;

	for (n=max_paths; n<p_paths->size; n++)
		free(p_paths->ppaths[n].par);
	p_paths->size = __min(max_paths,p_paths->size);
	p_paths->ppaths = realloc(p_paths->ppaths,p_paths->size*sizeof((*p_paths->ppaths)));
}

void TruncateGraphPaths(sized_paths **graph_paths,unsigned long nodes,unsigned long max_paths)
{	
	unsigned long y,x;
	
	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			TruncateSizedPaths(&(graph_paths[y][x]),max_paths);
}

sized_paths **CreateEmptyGraphPaths(unsigned long nodes)
{	
	sized_paths **graph_paths;
	unsigned long y,x;

	graph_paths = (sized_paths **) malloc(nodes*sizeof(sized_paths *));
	for (y=0; y<nodes; y++)
	{	graph_paths[y] = (sized_paths *) malloc(nodes*sizeof(sized_paths));
		for (x=0; x<nodes; x++)
		{	graph_paths[y][x].ppaths = NULL;
			graph_paths[y][x].size = 0;
		}
	}
	return graph_paths;
}


long CalcPathCost(unsigned long y,long **graph, path dpath)
{	unsigned long n,y1,x1;
	long cost = 0;

	x1 = y;
	for (n=0; n<dpath.size; n++)
	{	y1 = x1; x1 = dpath.par[n];
		cost += graph[y1][x1];
	}
	return cost;
}

void ClonePath(const path *spath, path *dpath)
{	// You should make sure that dpath.par does not point to an array before calling 
	// this function because it doesn't free the array.

	dpath->size = spath->size;
	dpath->tot_cost = spath->tot_cost;
	dpath->par = malloc((dpath->size+EXTRA)*sizeof(*(dpath->par)));
	memcpy(dpath->par,spath->par,(dpath->size+EXTRA)*sizeof(*(dpath->par)));
}

void FreeSizedPaths(sized_paths *p_paths)
{
	unsigned long n;

	for (n=0; n<p_paths->size; n++)
		free(p_paths->ppaths[n].par);
	free(p_paths->ppaths);
}

void FreeGraphPaths(sized_paths **graph_paths,unsigned long nodes)
{	
	unsigned long y,x;

	for (y=0; y<nodes; y++)
		for (x=0; x<nodes; x++)
			FreeSizedPaths(&(graph_paths[y][x]));
}

