/* This file is part of the Hegons.c file. Hegons.c will #include it.
   it is not to be compiled separately. It's been isolated from Hegons.c just for
   easy reading and convenience. MAKE files should not compile it.
*/

#define MIXED_RWA
ind_list_item *ptemp;


#ifdef MIXED_RWA

// Variables for use by the path routing functions below
unsigned long cux,cuy,path_num;

// Function pointers for path routing functions
unsigned long (*ChooseFirstPath)(unsigned long y,unsigned long x);
unsigned long (*ChoosePath)();
void (*UpdatePathSuccess)(unsigned long req_slots);


// First Fit Path Routing --------------------------------------
unsigned long FFChooseFirstPath(unsigned long y,unsigned long x)
{	cux=x; cuy=y;
	path_num=0; return 0;
}

unsigned long FFChoosePath()
{	
	path_num++; 
	if (path_num<gpaths[cuy][cux].size) 
		return path_num; else return INVALID;
}

void FFUpdatePathSuccess(unsigned long req_slots)
{	// Do Nothing!!
}


// Random Path Routing -------------------------------------------
unsigned long RandChooseFirstPath(unsigned long y,unsigned long x)
{	cux=x; cuy=y;

	GenRandomList(0,gpaths[cuy][cux].size,&rlist2);
	return GetNextInRList(&rlist2);
}

unsigned long RandChoosePath()
{	
	if (rlist2.r_n<rlist2.r_len) return GetNextInRList(&rlist2);
	else return INVALID;
}

void RandUpdatePathSuccess(unsigned long req_slots)
{	// Do Nothing!!
}



// Least Used & Most Used Path Routing --------------------------------------
unsigned long LUMUChooseFirstPath(unsigned long y,unsigned long x)
{	
	cux=x; cuy=y;
	ptemp = pr_path_data[cuy][cux].pfirst; return ptemp->item;
}

unsigned long LUMUChoosePath()
{
	if (ptemp->link!=NULL) { ptemp=ptemp->link; return ptemp->item; } 
	else return INVALID;
}

void LUMUUpdatePathSuccess(unsigned long req_slots)
{	
	cu_path_num = ptemp->item;
	IncreaseRListItem(&(pr_path_data[cuy][cux]),&(pr_path_data[cuy][cux].plist[cu_path_num]),req_slots);
}


#ifdef USE_LLML_R
/*	Least Loaded & Most Loaded Routing --------------------------------------
	This is not yet well defined for grooming case.
	If MIXED_RWA is not defined, these functions do not have a non-mixed alternative
	So MIXED_RWA has to be defined to use LL or ML routing 
*/
unsigned long *ptemp2=NULL;
unsigned long *ptemp3=NULL;

int (*CompareLLML)(const unsigned long *pn1,const unsigned long *pn2);

int AscCompareLLML(const unsigned long *pn1,const unsigned long *pn2)
{	
	// Sort in ascending order.
	return ptemp2[*pn1] - ptemp2[*pn2];
}

int DesCompareLLML(const unsigned long *pn1,const unsigned long *pn2)
{	
	// Sort in descending order.
	return ptemp2[*pn2] - ptemp2[*pn1];
}

unsigned long LLMLChooseFirstPath(unsigned long y,unsigned long x)
{	unsigned long n,n1;

	cux=x; cuy=y;

	// Each element will contain #of slots available in max loaded link in each path
	free(ptemp2);
	ptemp2 = malloc(sizeof(*ptemp2)*gpaths[cuy][cux].size);
	
	// Path numbers that will be sorted
	free(ptemp3);
	ptemp3 = malloc(sizeof(*ptemp3)*gpaths[cuy][cux].size);
	
	for (n1=0; n1<gpaths[cuy][cux].size; n1++)
	{	ptemp2[n1]=UINFINITY; ptemp3[n1]=n1;
		y=cuy; n=0; x=gpaths[cuy][cux].ppaths[n1].par[n];
		do
		{	
			ptemp2[n1] = __min(ptemp2[n1],chstate[y][x].tots_free);
			y=x; n++; x=gpaths[cuy][cux].ppaths[n1].par[n];
		} while IN_PATH(x,n);
	}
	
	// Sort
	qsort(ptemp3,gpaths[cuy][cux].size,sizeof(*ptemp3),CompareLLML);
	path_num=0; 
	return ptemp3[0];
}

unsigned long LLMLChoosePath()
{	
	path_num++; 
	if (path_num<gpaths[cuy][cux].size) 
		return ptemp3[path_num]; else return INVALID;
}

void LLMLUpdatePathSuccess(unsigned long req_slots)
{	// Do Nothing!!
}
#endif


// ---------------------------------------------------------------------------------------
// If mixed RWA is not used define all the above functions as macros
#else


// First Fit Path Routing --------------------------------------
#define FFChooseFirstPath(cy,cx) 0


#define FFChoosePath() \
( path_n++, ((path_n<gpaths[y][x].size) ? path_n : INVALID) )



#define FFUpdatePathSuccess(req_slots)


// Random Path Routing -------------------------------------------
#define RandChooseFirstPath(cy,cx) \
( GenRandomList(0,gpaths[cy][cx].size,&rlist2), GetNextInRList(&rlist2) )


#define RandChoosePath() \
	((rlist2.r_n<rlist2.r_len) ? GetNextInRList(&rlist2) : INVALID)


#define RandUpdatePathSuccess(req_slots)



// Most Used / Least Used Path Routing --------------------------------------
#define LUMUChooseFirstPath(cy,cx) \
( ptemp = pr_path_data[cy][cx].pfirst, ptemp->item )

#define LUMUChoosePath() \
( (ptemp->link!=NULL) ? (ptemp=ptemp->link, ptemp->item) : INVALID )


#define LUMUUpdatePathSuccess(req_slots) \
{	cu_path_num = ptemp->item; \
	IncreaseRListItem(&(pr_path_data[y][x]),&(pr_path_data[y][x].plist[cu_path_num]),req_slots); \
}

#endif //MIXED_RWA





